
<cfcomponent
	extends="BaseAPI"
	output="false"
	hint="I provide the Groups API.">
	
	
	<cffunction 
		name="DeleteGroup" 
		access="remote" 
		returntype="struct" 
		returnformat="json"
		output="false" 
		hint="I delete the given group.">
	
		<!--- Define arguments. --->
		<cfargument 
			name="ID" 
			type="numeric" 
			required="true" 
			hint="I am the ID of the group being deleted." 
			/>
		
		<!--- Define the local scope. --->
		<cfset var LOCAL = {} />
		
		<!--- Create a new response. --->
		<cfset LOCAL.Response = NewAPIResponse() />
		
		<!--- Try to execute. --->
		<cftry>
		
			<!--- Get the group. --->
			<cfset LOCAL.Group = VARIABLES.Factory.Get( "GroupService" ).Get( ARGUMENTS.ID ) />
		
			<!--- Delete the group. --->
			<cfset VARIABLES.Factory.Get( "GroupService" ).Delete( LOCAL.Group.ID ) />
			
			<!--- Store group as return data. --->
			<cfset LOCAL.Response.Data = LOCAL.Group />
		
			<!--- Catch any exceptions. --->
			<cfcatch>
				
				<!--- Update response. --->
				<cfset THIS.HandleAPIError( LOCAL.Response, CFCATCH ) />
			
			</cfcatch>			
		</cftry>	
				
		<!--- Return response. --->
		<cfreturn LOCAL.Response />
	</cffunction>
	
	
	<cffunction 
		name="GetGroup" 
		access="remote" 
		returntype="struct" 
		returnformat="json"
		output="false" 
		hint="I get the given group information.">
	
		<!--- Define arguments. --->
		<cfargument 
			name="ID" 
			type="numeric" 
			required="true" 
			hint="I am the ID of the group being gotten." 
			/>
		
		<!--- Define the local scope. --->
		<cfset var LOCAL = {} />
		
		<!--- Create a new response. --->
		<cfset LOCAL.Response = NewAPIResponse() />
		
		<cftry>
			
			<!--- Get the group. --->
			<cfset LOCAL.Group = VARIABLES.Factory.Get( "GroupService" ).Get( ARGUMENTS.ID ) />
						
			<!--- Store group as return data. --->
			<cfset LOCAL.Response.Data = LOCAL.Group />
			
			<!--- Catch any exceptions. --->
			<cfcatch>
				
				<!--- Update response. --->
				<cfset THIS.HandleAPIError( LOCAL.Response, CFCATCH ) />
				
			</cfcatch>			
		</cftry>	
		
		<!--- Return response. --->
		<cfreturn LOCAL.Response />
	</cffunction>
	
	
	<cffunction 
		name="GetGroups" 
		access="remote" 
		returntype="struct" 
		returnformat="json"
		output="false" 
		hint="I get the groups.">
	
		<!--- Define the local scope. --->
		<cfset var LOCAL = {} />
		
		<!--- Create a new response. --->
		<cfset LOCAL.Response = NewAPIResponse() />
		
		<cftry>
			
			<!--- Get all the groups. --->
			<cfset LOCAL.Response.Data = VARIABLES.Factory.Get( "GroupService" ).GetAll() />
			
			<!--- Catch any exceptions. --->
			<cfcatch>
				
				<!--- Update response. --->
				<cfset THIS.HandleAPIError( LOCAL.Response, CFCATCH ) />
					
			</cfcatch>			
		</cftry>	
		
		<!--- Return response. --->
		<cfreturn LOCAL.Response />
	</cffunction>
	
	
	<cffunction 
		name="SaveGroup" 
		access="remote" 
		returntype="struct" 
		returnformat="json"
		output="false" 
		hint="I save the given group.">
	
		<!--- Define arguments. --->
		<cfargument 
			name="ID" 
			type="numeric" 
			required="true" 
			hint="I am the ID of the group being saved." 
			/>
		
		<cfargument 
			name="Name" 
			type="string" 
			required="true" 
			hint="I am the name of the group."
			/>
			
		<cfargument 
			name="IsRemovedFromGeneralTweets" 
			type="boolean" 
			required="true" 
			hint="I flag whether of not these items will be removed from the general tweets when files."
			/>
	
		<cfargument 
			name="IncludePatterns" 
			type="string" 
			required="true" 
			hint="I am the list of patterns to define what gets included in this group."
			/>
			
		<cfargument 
			name="ExcludePatterns" 
			type="string" 
			required="true" 
			hint="I am the list of patterns to define what gets excluded from this group."
			/>
		
		<!--- Define the local scope. --->
		<cfset var LOCAL = {} />
		
		<!--- Create a new response. --->
		<cfset LOCAL.Response = NewAPIResponse() />
		
		
		<!--- Check to see if the data is valid. --->
		<cfif NOT Len( ARGUMENTS.Name )>
			
			<cfset ArrayAppend(
				LOCAL.Response.Errors,
				"Please enter a name."
				) />
			
		</cfif>
		
		
		<!--- Check to see if we have any errors. --->
		<cfif NOT ArrayLen( LOCAL.Response.Errors )>
	
			<!--- Try to execute. --->
			<cftry>
			
				<!--- Try to get group. --->
				<cftry>
					<cfset LOCAL.Group = VARIABLES.Factory.Get( "GroupService" ).Get( ARGUMENTS.ID ) />
					
					<!--- If group did not exist, get a new one. --->
					<cfcatch>
				
						<cfset LOCAL.Group = VARIABLES.Factory.Get( "GroupService" ).New() />
				
					</cfcatch>
				</cftry>
			
				
				<!--- Set values. --->
				<cfset LOCAL.Group.Name = ARGUMENTS.Name />
				<cfset LOCAL.Group.IsRemovedFromGeneralTweets = ARGUMENTS.IsRemovedFromGeneralTweets />
				<cfset LOCAL.Group.IncludePatterns = ARGUMENTS.IncludePatterns />
				<cfset LOCAL.Group.ExcludePatterns = ARGUMENTS.ExcludePatterns />
			
				<!--- Save the group. --->
				<cfset VARIABLES.Factory.Get( "GroupService" ).Save( LOCAL.Group ) />
				
				
				<!--- Store group as return data. --->
				<cfset LOCAL.Response.Data = LOCAL.Group />
			
				
				<!--- Catch any exceptions. --->
				<cfcatch>
					
					<!--- Update response. --->
					<cfset THIS.HandleAPIError( LOCAL.Response, CFCATCH ) />
						
				</cfcatch>			
			</cftry>	
			
		<cfelse>
		
			<!--- There were errors, so flag. --->
			<cfset LOCAL.Response.Success = false />
		
		</cfif>
		
		<!--- Return response. --->
		<cfreturn LOCAL.Response />
	</cffunction>
	
</cfcomponent>