
<cfcomponent
	extends="BaseAPI"
	output="false"
	hint="I provide Status API.">
	
	
	<cffunction 
		name="GetStatuses" 
		access="remote" 
		returntype="struct" 
		returnformat="json"
		output="false" 
		hint="I return a struct containing recent status keyed by group.">
		
		<!--- Define arguments. --->
		<cfargument 
			name="LastStatusID" 
			type="numeric" 
			required="false" 
			default="0"
			hint="I am the ID of the last status ID returned."
			/>
		
		<!--- Define the local scope. --->
		<cfset var LOCAL = {} />
		
		<!--- Create a new response. --->
		<cfset LOCAL.Response = NewAPIResponse() />
		
		
		<!--- Launch parallel thread to update the database. --->
		<cfthread
			action="run"
			name="#CreateUUID()#">
			
			<!--- Update database. --->			
			<cfset VARIABLES.Factory.Get( "TwitterService" ).UpdateStatuses() />
			
		</cfthread>
		
		
		<cftry>
		
			<!--- 
				Create the response object. The response data will be categorized by 
				the type of category. Within each will be an array of statuses.
			--->
			<cfset LOCAL.Response.Data = {
				General = [],
				Replies = [],
				Direct = [],
				Snoozed = []
				} />
			
					
			<!--- Reset the snoozing. --->
			<cfset VARIABLES.Factory.Get( "FriendService" ).ResetSnooze() />
					
			<!--- Get all the recent statuses.  --->
			<cfset LOCAL.Statuses = VARIABLES.Factory.Get( "StatusService" ).GetRecent( ARGUMENTS.LastStatusID ) />
		
		
			<!--- Loop over the statuses to orgranize them. --->
			<cfloop 
				index="LOCAL.Status"
				array="#LOCAL.Statuses#">
			
				<!--- Get the friend for this status. --->
				<cfset LOCAL.Friend = VARIABLES.Factory.Get( "FriendService" ).Get( LOCAL.Status.FriendID ) />
			
			
				<!--- Create a default return status. --->
				<cfset LOCAL.ReturnStatus = {
					<!--- Status data. --->
					id = LOCAL.Status.ID,
					status = LOCAL.Status.Status,
					is_general = LOCAL.Status.IsGeneral,
					is_reply = LOCAL.Status.IsReply,
					is_direct = LOCAL.Status.IsDirect,
					is_snoozed = LOCAL.Status.IsSnoozed,
					date_created = LOCAL.Status.DateCreated,
					friend_id = LOCAL.Status.FriendID,
					status_group_id = LOCAL.Status.GroupID,
					
					<!--- Author. --->
					authored = (
						LOCAL.Friend.Name & 
						" posted @ " & 
						DateFormat( LOCAL.Status.DateCreated, "mmm d, yyyy" ) & 
						" at " & 
						TimeFormat( LOCAL.Status.DateCreated, "h:mm TT" )
						),
						
					<!--- Friend data. --->
					friend_name = LOCAL.Friend.Name,
					friend_screen_name = LOCAL.Friend.ScreenName,
					friend_photo_url = LOCAL.Friend.PhotoURL,
					friend_date_snoozed_until = LOCAL.Friend.DateSnoozedUntil,
					friend_is_snoozed = 0,
					snooze_time = ""
					} />
			
			
				<!--- Check to see if this friend is being snoozed. --->
				<cfif IsDate( LOCAL.Friend.DateSnoozedUntil )>
				
					<!--- Set the snoozed flag. --->
					<cfset LOCAL.ReturnStatus.friend_is_snoozed = true />
				
					<!--- Update the time for snoozing. --->
					<cfset LOCAL.ReturnStatus.snooze_time = (
						DateDiff(
							"n",
							Now(),
							LOCAL.Friend.DateSnoozedUntil
							) &
						" Mins"								
						) />				
				
				</cfif>
			
				
				<!--- Check to see if this status has a group. --->
				<cfif LOCAL.ReturnStatus.status_group_id>
					
					<!--- This is a custom group. --->
					
					<!--- Check to see if this group has a response key yet. --->
					<cfif NOT StructKeyExists( LOCAL.Response.Data, LOCAL.ReturnStatus.status_group_id )>
						
						<!--- Create empyt array. --->
						<cfset LOCAL.Response.Data[ LOCAL.ReturnStatus.status_group_id ] = [] />
					
					</cfif>
					
					<!--- Add the query to reponse. --->				
					<cfset ArrayAppend( 
						LOCAL.Response.Data[ LOCAL.ReturnStatus.status_group_id ],
						LOCAL.ReturnStatus
						) />
				
				<cfelse>
				
					<!--- This is a built-in group. --->
					
					<!--- We need to add this to the right group. Let's get the right key. --->
					<cfif LOCAL.ReturnStatus.is_reply>
						
						<cfset LOCAL.GroupKey = "Replies" />
						
					<cfelseif LOCAL.ReturnStatus.is_direct>
						
						<cfset LOCAL.GroupKey = "Direct" />
						
					<cfelseif LOCAL.ReturnStatus.is_snoozed>
						
						<cfset LOCAL.GroupKey = "Snoozed" />
												
					<cfelse>
					
						<!--- By default use general group. --->
						<cfset LOCAL.GroupKey = "General" />
						
					</cfif>
					
					<!--- Add the query to reponse. --->				
					<cfset ArrayAppend( 
						LOCAL.Response.Data[ LOCAL.GroupKey ],
						LOCAL.ReturnStatus
						) />
					
				</cfif>
			
			</cfloop>
		
			
			<!--- Catch any exceptions. --->
			<cfcatch>
				
				<!--- Update response. --->
				<cfset THIS.HandleAPIError( LOCAL.Response, CFCATCH ) />
				
			</cfcatch>			
		</cftry>	
		
		<!--- Return response. --->
		<cfreturn LOCAL.Response />
	</cffunction>
	
	
	<cffunction 
		name="SubmitStatus" 
		access="remote" 
		returntype="struct"
		returnformat="json"
		output="false" 
		hint="I submit a status message to Twitter.">
	
		<!--- Define arguments. --->
		<cfargument 
			name="Status"
			type="string" 
			required="true" 
			hint="I am the status being submitted to Twitter."
			/>
		
		
		<!--- Define the local scope. --->
		<cfset var LOCAL = {} />
		
		<!--- Create a new response. --->
		<cfset LOCAL.Response = NewAPIResponse() />
		
		<cftry>
		
			<!--- Submit the status. --->
			<cfset VARIABLES.Factory.Get( "TwitterService" ).SubmitStatus( ARGUMENTS.Status ) />
		
			<!--- Catch any exceptions. --->
			<cfcatch>
				
				<!--- Update response. --->
				<cfset THIS.HandleAPIError( LOCAL.Response, CFCATCH ) />
					
			</cfcatch>			
		</cftry>	
		
		<!--- Return response. --->
		<cfreturn LOCAL.Response />
	</cffunction>
	
</cfcomponent>