
<cfcomponent
	output="false"
	hint="I provide base service class functionality.">
	
	
	<cffunction 
		name="QueryRowToStruct" 
		access="private" 
		returntype="struct" 
		output="false" 
		hint="I turn the given query row into a struct.">
		
		<!--- Define the arguments. --->
		<cfargument 
			name="Query" 
			type="query" 
			required="true" 
			hint="I am the query we are using."
			/>
			
		<cfargument 
			name="Row" 
			type="numeric" 
			required="false" 
			default="#ARGUMENTS.Query.CurrentRow#"
			hint="I am the index of the row we are converting to a query. I default to the current row of the contextual query loop."
			/>
		
		<!--- Define the local scope. --->
		<cfset var LOCAL = {} />
		
		
		<!--- Check to make sure that the record was found. --->
		<cfif (
			(NOT ARGUMENTS.Row) OR
			(ARGUMENTS.Query.RecordCount LT ARGUMENTS.Row)
			)>
			
			<!--- The record was not found, throw an error. --->
			<cfthrow
				type="KinkyTwits.BaseService.QueryRowToStruct.RowNotFound"
				message="The given row cannot be found in this query."
				detail="The given row, #ARGUMENTS.Row#, cannot be found in this query which has only #ARGUMENTS.Query.RecordCount# record(s)."
				/>
		
		</cfif>
		
		
		<!--- Create the base structure we are going to return. --->
		<cfset LOCAL.RowStruct = {} />
		
		<!--- Loop over the column to create the struct. --->
		<cfloop
			index="LOCAL.ColumnName"
			list="#ARGUMENTS.Query.ColumnList#"
			delimiters=",">
			
			<!--- Move the data field into the struct using the same name. --->
			<cfset LOCAL.RowStruct[ LOCAL.ColumnName ] = ARGUMENTS.Query[ LOCAL.ColumnName ][ ARGUMENTS.Row ] />
			
		</cfloop>
		
		<!--- Return the populated row structure. --->
		<cfreturn LOCAL.RowStruct />
	</cffunction>
	
	
	<cffunction 
		name="QueryToArray" 
		access="private" 
		returntype="array" 
		output="false" 
		hint="I turn the given query to an array of structures.">
		
		<!--- Define arguments. --->
		<cfargument 
			name="Query" 
			type="query" 
			required="true" 
			hint="I am the query being turned into an array."
			/>
		
		<!--- Define the local scope. --->
		<cfset var LOCAL = {} />
		
		<!--- Define the base array for our query. --->
		<cfset LOCAL.QueryArray = [] />
		
		<!--- Loop over the query and add each row as its struct-equivalent to the query array. --->
		<cfloop query="ARGUMENTS.Query">
			
			<cfset ArrayAppend(
				LOCAL.QueryArray,
				VARIABLES.QueryRowToStruct( ARGUMENTS.Query )
				) />
		
		</cfloop>
				
		<!--- Return populated array. --->
		<cfreturn LOCAL.QueryArray />
	</cffunction>
	
</cfcomponent>