
<cfcomponent
	output="false"
	hint="I provide create object functionality.">
	
	
	<cffunction 
		name="Init" 
		access="public"
		returntype="any" 
		output="false" 
		hint="I return an intialized object.">
	
		<!--- Define arguments. --->
		<cfargument 
			name="DataFolder" 
			type="string" 
			required="true" 
			hint="The path to the data folder." 
			/>
			
		<cfargument 
			name="LoggingFolder" 
			type="string" 
			required="true" 
			hint="The path to the logging folder." 
			/>
			
		<cfargument 
			name="Username" 
			type="string" 
			required="true" 
			hint="The Twitter account username." 
			/>
			
		<cfargument 
			name="Password" 
			type="string" 
			required="true" 
			hint="The Twitter account username." 
			/>
		
		<!--- Store the data folder. --->
		<cfset VARIABLES.DataFolder = ARGUMENTS.DataFolder />
		<cfset VARIABLES.LoggingFolder = ARGUMENTS.LoggingFolder />
		
		<!--- Create singleton cache. --->
		<cfset VARIABLES.Cache = {} />
		
		<!--- Create singletons. --->
		<cfset VARIABLES.Cache.StatusService = THIS.CFC( "StatusService" ).Init(
			Factory = THIS,
			DataFolder = VARIABLES.DataFolder
			) />
			
		<cfset VARIABLES.Cache.GroupService = THIS.CFC( "GroupService" ).Init(
			Factory = THIS,
			DataFolder = VARIABLES.DataFolder
			) />
			
		<cfset VARIABLES.Cache.FriendService = THIS.CFC( "FriendService" ).Init(
			Factory = THIS,
			DataFolder = VARIABLES.DataFolder
			) />
			
		<cfset VARIABLES.Cache.TwitterService = THIS.CFC( "TwitterService" ).Init(
			Factory = THIS,
			Username = ARGUMENTS.Username,
			Password = ARGUMENTS.Password
			) />
			
		<cfset VARIABLES.Cache.URLService = THIS.CFC( "TinyURLService" ).Init(
			Factory = THIS
			) />
			
		<cfset VARIABLES.Cache.LoggingService = THIS.CFC( "LoggingService" ).Init(
			Factory = THIS,
			LoggingFolder = VARIABLES.LoggingFolder
			) />
		
		<!--- Return This reference. --->
		<cfreturn THIS />
	</cffunction>
	

	<cffunction 
		name="CFC" 
		access="public" 
		returntype="any" 
		output="false" 
		hint="I simply the CFC creation.">
		
		<!--- Define arguments. --->	
		<cfargument 
			name="CFCName" 
			type="string" 
			required="true" 
			hint="I am the name of the CFC class."
			/>
			
		<!--- Return non-initialized object. --->
		<cfreturn CreateObject( "component", ARGUMENTS.CFCName ) />
	</cffunction>
	
	
	<cffunction 
		name="Get" 
		access="public" 
		returntype="any" 
		output="false" 
		hint="I return the given object.">
		
		<!--- Define arguments. --->
		<cfargument 
			name="Type" 
			type="string" 
			required="true" 
			hint="The type of CFC to create." 
			/>
			
		<!--- Check to see if it is a cached type. --->
		<cfif StructKeyExists( VARIABLES.Cache, ARGUMENTS.Type )>
			
			<!--- Return cached singleton. --->
			<cfreturn VARIABLES.Cache[ ARGUMENTS.Type ] />
		
		</cfif>
		
		<!--- If nothing was returned yet, try to create the CFC manually. --->
		<cfreturn THIS.CFC( ARGUMENTS.Type ) />		
	</cffunction>

</cfcomponent>