
<cfcomponent
	extends="BaseService"
	output="false"
	hint="I provide service methods for Friend objects.">

	
	<cffunction 
		name="Init" 
		access="public" 
		returntype="any" 
		output="false" 
		hint="I return an intialized object.">
		
		<!--- Define arguments. --->
		<cfargument 
			name="Factory" 
			type="any" 
			required="true" 
			hint="I am the applications object factory."
			/>
		
		<cfargument 
			name="DataFolder" 
			type="string" 
			required="true" 
			hint="I am the data folder for this application."
			/>
		
		<!--- Store the factory. --->
		<cfset VARIABLES.Factory = ARGUMENTS.Factory />
		
		<!--- Store the data folder. --->
		<cfset VARIABLES.DataFolder = ARGUMENTS.DataFolder />
		
		<!--- Store the file path. --->
		<cfset VARIABLES.DataFile = (VARIABLES.DataFolder & "friends.json") />
		
		<!--- Set up instance variables. --->
		<cfset VARIABLES.Instance = {
			Friends = QueryNew(
				"ID,TwitterID,Name,ScreenName,PhotoURL,TwitterURL,DateSnoozedUntil,DateCreated",
				"cf_sql_integer,cf_sql_integer,cf_sql_varchar,cf_sql_varchar,cf_sql_varchar,cf_sql_varchar,cf_sql_timestamp,cf_sql_timestamp"
				),
			IDIndex = 0
			} />
			
		<!--- Load the friends data from file. --->	
		<cfset VARIABLES.LoadFriendData() />
		
		<!--- Return This reference. --->
		<cfreturn THIS />
	</cffunction>
	
	
	<cffunction 
		name="Delete" 
		access="public" 
		returntype="struct" 
		output="false" 
		hint="I delete the given Friend at the given ID.">
		
		<!--- Define arguments. --->
		<cfargument 
			name="ID" 
			type="numeric" 
			required="true" 
			hint="I am the internal ID of the given friend."
			/>
			
		<!--- Define the local scope. --->
		<cfset var LOCAL = {} />
		
		<!--- Get the given friend. --->
		<cfset LOCAL.Friend = THIS.Get( ARGUMENTS.ID ) />
		
		<!--- Delete the given record. --->
		<cfquery name="VARIABLES.Instance.Friends" dbtype="query">
			SELECT 
				* 
			FROM
				VARIABLES.Instance.Friends
			WHERE
				ID != <cfqueryparam value="#ARGUMENTS.ID#" cfsqltype="cf_sql_integer" />
			ORDER BY
				ScreenName ASC
		</cfquery>
		
		<!--- Persist the friend data. --->
		<cfset VARIABLES.PersistFriendData() />
		
		<!--- Delete all statuses for this friend. --->
		<cfset VARIABLES.Factory.Get( "StatusService" ).DeleteByFriend( LOCAL.Friend ) />
		
		<!--- Return the Friend. --->
		<cfreturn LOCAL.Friend />
	</cffunction>
	
	
	<cffunction 
		name="Get" 
		access="public" 
		returntype="struct" 
		output="false" 
		hint="I return the given Friend at the given ID.">
		
		<!--- Define arguments. --->
		<cfargument 
			name="ID" 
			type="numeric" 
			required="true" 
			hint="I am the internal ID of the given friend."
			/>
			
		<!--- Define the local scope. --->
		<cfset var LOCAL = {} />
		
		<!--- Query for the current row. --->
		<cfquery name="LOCAL.FriendData" dbtype="query">
			SELECT
				*
			FROM
				VARIABLES.Instance.Friends
			WHERE
				ID = <cfqueryparam value="#ARGUMENTS.ID#" cfsqltype="cf_sql_integer" />
		</cfquery>
		
		<!--- Create the intermediar friend sturct. --->
		<cfset LOCAL.Friend = VARIABLES.QueryRowToStruct( LOCAL.FriendData ) />
		
		<!--- Return the Friend. --->
		<cfreturn LOCAL.Friend />
	</cffunction>
	
	
	<cffunction 
		name="GetAll" 
		access="public" 
		returntype="array" 
		output="false" 
		hint="I return the collection of Friends.">
			
		<!--- Return the Friend. --->
		<cfreturn VARIABLES.QueryToArray( VARIABLES.Instance.Friends ) />
	</cffunction>
	
	
	<cffunction 
		name="GetByTwitterID" 
		access="public" 
		returntype="struct" 
		output="false" 
		hint="I return the friend with the given Twitter ID.">
	
		<!--- Define arguments. --->
		<cfargument 
			name="TwitterID" 
			type="numeric" 
			required="true" 
			hint="I am the Twitter ID of the friend."
			/>
			
		<!--- Define the local scope. --->
		<cfset var LOCAL = {} />
		
		<!--- Query for friend. --->
		<cfquery name="LOCAL.FriendData" dbtype="query">
			SELECT
				*
			FROM
				VARIABLES.Instance.Friends
			WHERE
				TwitterID = <cfqueryparam value="#ARGUMENTS.TwitterID#" cfsqltype="cf_sql_integer" />
		</cfquery>
		
		
		<!--- Return friend. --->
		<cfreturn VARIABLES.QueryRowToStruct( LOCAL.FriendData ) />
	</cffunction>
	
	
	<cffunction 
		name="LoadFriendData" 
		access="private" 
		returntype="void" 
		output="false" 
		hint="I load the friend data file from disk.">
		
		<!--- Define the local scope. --->
		<cfset var LOCAL = {} />
		
		<!--- Try to load the file data - it might not exist yet. --->
		<cftry>
		
			<!--- Load the friend JSON file. --->
			<cffile
				action="read"
				file="#VARIABLES.DataFile#"
				variable="LOCAL.FriendData"
				/>
			
			<!--- Deserialize the JSON and store internally as an array. --->
			<cfset LOCAL.Friends = DeserializeJSON( LOCAL.FriendData ) />
			
			<!--- Catch any errors. --->
			<cfcatch>
				
				<!--- 
					The file did not exist or there was an error in the JSON data. 
					Just create a default structure as our disaster recovery plan.
				--->
				<cfset LOCAL.Friends = [] />
				
			</cfcatch>
		</cftry>
		
		
		<!--- 
			Now that we have the friends array, we need to convert that to out 
			internal query structure.
		--->
		<cfloop
			index="LOCAL.Friend"
			array="#LOCAL.Friends#">
				
			<!--- Add a query row. --->
			<cfset QueryAddRow( VARIABLES.Instance.Friends ) />
			
			<!--- Set column values. --->
			<cfset VARIABLES.Instance.Friends[ "ID" ][ VARIABLES.Instance.Friends.RecordCount ] = JavaCast( "int", LOCAL.Friend.ID ) />
			<cfset VARIABLES.Instance.Friends[ "TwitterID" ][ VARIABLES.Instance.Friends.RecordCount ] = JavaCast( "int", LOCAL.Friend.TwitterID ) />
			<cfset VARIABLES.Instance.Friends[ "Name" ][ VARIABLES.Instance.Friends.RecordCount ] = JavaCast( "string", LOCAL.Friend.Name ) />
			<cfset VARIABLES.Instance.Friends[ "ScreenName" ][ VARIABLES.Instance.Friends.RecordCount ] = JavaCast( "string", LOCAL.Friend.ScreenName ) />
			<cfset VARIABLES.Instance.Friends[ "PhotoURL" ][ VARIABLES.Instance.Friends.RecordCount ] = JavaCast( "string", LOCAL.Friend.PhotoURL ) />
			<cfset VARIABLES.Instance.Friends[ "TwitterURL" ][ VARIABLES.Instance.Friends.RecordCount ] = JavaCast( "string", LOCAL.Friend.TwitterURL ) />
			<cfif IsDate( LOCAL.Friend.DateSnoozedUntil )>
				<cfset VARIABLES.Instance.Friends[ "DateSnoozedUntil" ][ VARIABLES.Instance.Friends.RecordCount ] = ParseDateTime( LOCAL.Friend.DateSnoozedUntil ) />
			<cfelse>
				<cfset VARIABLES.Instance.Friends[ "DateSnoozedUntil" ][ VARIABLES.Instance.Friends.RecordCount ] = JavaCast( "null", 0 ) />
			</cfif>
			<cfset VARIABLES.Instance.Friends[ "DateCreated" ][ VARIABLES.Instance.Friends.RecordCount ] = ParseDateTime( LOCAL.Friend.DateCreated ) />

			<!--- Keep track of the max ID index (which will be used to generate new objects). --->
			<cfset VARIABLES.Instance.IDIndex = Max( VARIABLES.Instance.IDIndex, LOCAL.Friend.ID ) />
			
		</cfloop>
		
		
		<!--- Re-order them for proper name. --->
		<cfquery name="VARIABLES.Instance.Friends" dbtype="query">
			SELECT
				*
			FROM
				VARIABLES.Instance.Friends
			ORDER BY
				ScreenName ASC		
		</cfquery>
		
		
		<!--- Return out. --->
		<cfreturn />
	</cffunction>
	
	
	<cffunction 
		name="New" 
		access="public" 
		returntype="struct" 
		output="false" 
		hint="I create a new Friend object.">
	
		<!--- Define the local scope. --->
		<cfset var LOCAL = {} />
		
		<!--- Create a default Friend struct. --->
		<cfset LOCAL.Friend = {
			ID = (++VARIABLES.Instance.IDIndex),
			TwitterID = 0,
			Name = "",
			ScreeName = "",
			PhotoURL = "",
			TwitterURL = "",
			DateSnoozedUntil = "",
			DateCreated = Now()
			} />
			
		<!--- Return the new friend. --->
		<cfreturn LOCAL.Friend />
	</cffunction>
	
	
	<cffunction 
		name="PersistFriendData" 
		access="private" 
		returntype="void" 
		output="false" 
		hint="I persist the friend data file to disk.">
		
		<!--- Define the local scope. --->
		<cfset var LOCAL = {} />
		
		<!--- Write the friend JSON file. --->
		<cffile
			action="write"
			file="#VARIABLES.DataFile#"
			output="#SerializeJSON( VARIABLES.QueryToArray( VARIABLES.Instance.Friends ) )#"
			/>
		
		<!--- Return out. --->
		<cfreturn />
	</cffunction>
	
	
	<cffunction 
		name="ResetSnooze" 
		access="public" 
		returntype="void" 
		output="false" 
		hint="I reset the snooze values for all outdates snoozes.">
	
		<!--- Define the local scope. --->
		<cfset var LOCAL = {} />
		
		<!--- Query for all friends who need to be unsnoozed. --->
		<cfquery name="LOCAL.Friends" dbtype="query">
			SELECT
				ID
			FROM
				VARIABLES.Instance.Friends
			WHERE
				DateSnoozedUntil IS NOT NULL
			AND
				DateSnoozedUntil <= <cfqueryparam value="#Now()#" cfsqltype="cf_sql_timestamp" />
		</cfquery>
		
		<!--- Loop over the friends and try to unsnooze. --->
		<cfloop query="LOCAL.Friends">
			
			<!--- Try to do this (it might throw an exception if the friend cannot be found). --->
			<cftry>
				<!--- Remove the snooze. --->
				<cfset THIS.Unsnooze( LOCAL.Friends.ID ) />
			
				<!--- Catch any errors. --->
				<cfcatch>
					<!--- Friend did not exist. --->
				</cfcatch>
			</cftry>
		
		</cfloop>
		
		<!--- Return out. --->
		<cfreturn />
	</cffunction>
	
	
	<cffunction 
		name="Save" 
		access="public" 
		returntype="struct" 
		output="false" 
		hint="I save the given Friend object.">
	
		<!--- Define aruments. --->	
		<cfargument 
			name="Friend" 
			type="struct" 
			required="true" 
			hint="I am a Friend object returned from New()." 
			/>
		
		<!--- Remove any internal record temporarily. --->
		<cfquery name="VARIABLES.Instance.Friends" dbtype="query">
			SELECT
				*
			FROM
				VARIABLES.Instance.Friends
			WHERE
				ID != <cfqueryparam value="#ARGUMENTS.Friend.ID#" cfsqltype="cf_sql_integer" />		
		</cfquery>
		
		<!--- 
			Now that we have removed the old row, we can re-insert it with the 
			new and / or updated information.
		--->
		
		<!--- Add a query row. --->
		<cfset QueryAddRow( VARIABLES.Instance.Friends ) />
		
		<!--- Set column values. --->
		<cfset VARIABLES.Instance.Friends[ "ID" ][ VARIABLES.Instance.Friends.RecordCount ] = JavaCast( "int", ARGUMENTS.Friend.ID ) />
		<cfset VARIABLES.Instance.Friends[ "TwitterID" ][ VARIABLES.Instance.Friends.RecordCount ] = JavaCast( "int", ARGUMENTS.Friend.TwitterID ) />
		<cfset VARIABLES.Instance.Friends[ "Name" ][ VARIABLES.Instance.Friends.RecordCount ] = JavaCast( "string", ARGUMENTS.Friend.Name ) />
		<cfset VARIABLES.Instance.Friends[ "ScreenName" ][ VARIABLES.Instance.Friends.RecordCount ] = JavaCast( "string", ARGUMENTS.Friend.ScreenName ) />
		<cfset VARIABLES.Instance.Friends[ "PhotoURL" ][ VARIABLES.Instance.Friends.RecordCount ] = JavaCast( "string", ARGUMENTS.Friend.PhotoURL ) />
		<cfset VARIABLES.Instance.Friends[ "TwitterURL" ][ VARIABLES.Instance.Friends.RecordCount ] = JavaCast( "string", ARGUMENTS.Friend.TwitterURL ) />
		<cfif IsDate( ARGUMENTS.Friend.DateSnoozedUntil )>
			<cfset VARIABLES.Instance.Friends[ "DateSnoozedUntil" ][ VARIABLES.Instance.Friends.RecordCount ] = ParseDateTime( ARGUMENTS.Friend.DateSnoozedUntil ) />
		<cfelse>
			<cfset VARIABLES.Instance.Friends[ "DateSnoozedUntil" ][ VARIABLES.Instance.Friends.RecordCount ] = JavaCast( "null", 0 ) />
		</cfif>
		<cfset VARIABLES.Instance.Friends[ "DateCreated" ][ VARIABLES.Instance.Friends.RecordCount ] = ParseDateTime( ARGUMENTS.Friend.DateCreated ) />
		
		<!--- Update the file data. --->
		<cfset VARIABLES.PersistFriendData() />
		
		<!--- Return friend. --->
		<cfreturn ARGUMENTS.Friend />
	</cffunction>
	
	
	<cffunction 
		name="Snooze" 
		access="public" 
		returntype="struct" 
		output="false" 
		hint="I snooze the Friend at the given ID.">
		
		<!--- Define arguments. --->
		<cfargument 
			name="ID" 
			type="numeric" 
			required="true" 
			hint="I am the internal ID of the given friend."
			/>
			
		<!--- Define the local scope. --->
		<cfset var LOCAL = {} />
		
		<!--- Get the given friend. --->
		<cfset LOCAL.Friend = THIS.Get( ARGUMENTS.ID ) />
								
		<!--- Update the snooze. --->
		<cfif IsDate( LOCAL.Friend.DateSnoozedUntil )>
		
			<!--- Add an hour to the existing snooze time. --->
			<cfset LOCAL.Friend.DateSnoozedUntil = DateAdd( "h", 1, LOCAL.Friend.DateSnoozedUntil ) />
		
		<cfelse>
			
			<!--- Set snooze time for an hour. --->
			<cfset LOCAL.Friend.DateSnoozedUntil = DateAdd( "h", 1, Now() ) />
		
		</cfif>
		
		<!--- Save friend. --->
		<cfset THIS.Save( LOCAL.Friend ) />
		
		<!--- Return the Friend. --->
		<cfreturn LOCAL.Friend />
	</cffunction>
	
	
	<cffunction 
		name="Unsnooze" 
		access="public" 
		returntype="struct" 
		output="false" 
		hint="I unsnooze the Friend at the given ID.">
		
		<!--- Define arguments. --->
		<cfargument 
			name="ID" 
			type="numeric" 
			required="true" 
			hint="I am the internal ID of the given friend."
			/>
			
		<!--- Define the local scope. --->
		<cfset var LOCAL = {} />
		
		<!--- Get the given friend. --->
		<cfset LOCAL.Friend = THIS.Get( ARGUMENTS.ID ) />
		
		<!--- Reset the snooze. --->
		<cfset LOCAL.Friend.DateSnoozedUntil = "" />
		
		<!--- Save friend. --->
		<cfset THIS.Save( LOCAL.Friend ) />
		
		<!--- Return the Friend. --->
		<cfreturn LOCAL.Friend />
	</cffunction>
	
</cfcomponent>