
<cfcomponent
	extends="BaseService"
	output="false"
	hint="I provide service functions for groups.">

	
	<cffunction 
		name="Init" 
		access="public" 
		returntype="any" 
		output="false" 
		hint="I return an intialized object.">
		
		<!--- Define arguments. --->
		<cfargument 
			name="Factory" 
			type="any" 
			required="true" 
			hint="I am the applications object factory."
			/>
		
		<cfargument 
			name="DataFolder" 
			type="string" 
			required="true" 
			hint="I am the data folder for this application."
			/>
		
		<!--- Store the factory. --->
		<cfset VARIABLES.Factory = ARGUMENTS.Factory />
		
		<!--- Store the data folder. --->
		<cfset VARIABLES.DataFolder = ARGUMENTS.DataFolder />
		
		<!--- Store the file path. --->
		<cfset VARIABLES.DataFile = (VARIABLES.DataFolder & "groups.json") />
		
		<!--- Set up instance variables. --->
		<cfset VARIABLES.Instance = {
			Groups = QueryNew(
				"ID,Name,IsRemovedFromGeneralTweets,IncludePatterns,ExcludePatterns,DateCreated",
				"cf_sql_integer,cf_sql_varchar,cf_sql_bit,cf_sql_varchar,cf_sql_varchar,cf_sql_timestamp"
				),
			IDIndex = 0
			} />
			
		<!--- Load the groups data from file. --->	
		<cfset VARIABLES.LoadGroupData() />
		
		<!--- Return This reference. --->
		<cfreturn THIS />
	</cffunction>
	
	
	<cffunction 
		name="Delete" 
		access="public" 
		returntype="struct" 
		output="false" 
		hint="I delete the given Group at the given ID.">
		
		<!--- Define arguments. --->
		<cfargument 
			name="ID" 
			type="numeric" 
			required="true" 
			hint="I am the internal ID of the given group."
			/>
			
		<!--- Define the local scope. --->
		<cfset var LOCAL = {} />
		
		<!--- Get the given group. --->
		<cfset LOCAL.Group = THIS.Get( ARGUMENTS.ID ) />
				
		<!--- Delete the given record. --->
		<cfquery name="VARIABLES.Instance.Groups" dbtype="query">
			SELECT 
				* 
			FROM
				VARIABLES.Instance.Groups
			WHERE
				ID != <cfqueryparam value="#ARGUMENTS.ID#" cfsqltype="cf_sql_integer" />
			ORDER BY
				Name ASC
		</cfquery>
		
		<!--- Persist the group data. --->
		<cfset VARIABLES.PersistGroupData() />
		
		<!--- Delete all statuses in this group. --->
		<cfset VARIABLES.Factory.Get( "StatusService" ).DeleteByGroup( LOCAL.Group ) />
		
		<!--- Return the Group. --->
		<cfreturn LOCAL.Group />
	</cffunction>
	
	
	<cffunction 
		name="Get" 
		access="public" 
		returntype="struct" 
		output="false" 
		hint="I return the given Group at the given ID.">
		
		<!--- Define arguments. --->
		<cfargument 
			name="ID" 
			type="numeric" 
			required="true" 
			hint="I am the internal ID of the given group."
			/>
			
		<!--- Define the local scope. --->
		<cfset var LOCAL = {} />
		
		<!--- Query for the current row. --->
		<cfquery name="LOCAL.GroupData" dbtype="query">
			SELECT
				*
			FROM
				VARIABLES.Instance.Groups
			WHERE
				ID = <cfqueryparam value="#ARGUMENTS.ID#" cfsqltype="cf_sql_integer" />
		</cfquery>
		
		<!--- Create the intermediar group sturct. --->
		<cfset LOCAL.Group = VARIABLES.QueryRowToStruct( LOCAL.GroupData ) />
		
		<!--- Return the Group. --->
		<cfreturn LOCAL.Group />
	</cffunction>
	
	
	<cffunction 
		name="GetAll" 
		access="public" 
		returntype="array" 
		output="false" 
		hint="I return the array of Groups.">
			
		<!--- Return the Group. --->
		<cfreturn VARIABLES.QueryToArray( VARIABLES.Instance.Groups ) />
	</cffunction>
	
	
	<cffunction 
		name="LoadGroupData" 
		access="private" 
		returntype="void" 
		output="false" 
		hint="I load the group data file from disk.">
		
		<!--- Define the local scope. --->
		<cfset var LOCAL = {} />
		
		<!--- Try to load the file data - it might not exist yet. --->
		<cftry>
		
			<!--- Load the group JSON file. --->
			<cffile
				action="read"
				file="#VARIABLES.DataFile#"
				variable="LOCAL.GroupData"
				/>
			
			<!--- Deserialize the JSON and store internally as an array. --->
			<cfset LOCAL.Groups = DeserializeJSON( LOCAL.GroupData ) />
			
			<!--- Catch any errors. --->
			<cfcatch>
				
				<!--- 
					The file did not exist or there was an error in the JSON data. 
					Just create a default structure as our disaster recovery plan.
				--->
				<cfset LOCAL.Groups = [] />
				
			</cfcatch>
		</cftry>
		
		
		<!--- 
			Now that we have the groups array, we need to convert that to out 
			internal query structure.
		--->
		<cfloop
			index="LOCAL.Group"
			array="#LOCAL.Groups#">
				
			<!--- Add a query row. --->
			<cfset QueryAddRow( VARIABLES.Instance.Groups ) />
			
			<!--- Set column values. --->
			<cfset VARIABLES.Instance.Groups[ "ID" ][ VARIABLES.Instance.Groups.RecordCount ] = JavaCast( "int", LOCAL.Group.ID ) />
			<cfset VARIABLES.Instance.Groups[ "Name" ][ VARIABLES.Instance.Groups.RecordCount ] = JavaCast( "string", LOCAL.Group.Name ) />
			<cfset VARIABLES.Instance.Groups[ "IsRemovedFromGeneralTweets" ][ VARIABLES.Instance.Groups.RecordCount ] = JavaCast( "boolean", LOCAL.Group.IsRemovedFromGeneralTweets ) />
			<cfset VARIABLES.Instance.Groups[ "IncludePatterns" ][ VARIABLES.Instance.Groups.RecordCount ] = JavaCast( "string", LOCAL.Group.IncludePatterns ) />
			<cfset VARIABLES.Instance.Groups[ "ExcludePatterns" ][ VARIABLES.Instance.Groups.RecordCount ] = JavaCast( "string", LOCAL.Group.ExcludePatterns ) />
			<cfset VARIABLES.Instance.Groups[ "DateCreated" ][ VARIABLES.Instance.Groups.RecordCount ] = ParseDateTime( LOCAL.Group.DateCreated ) />
		
			<!--- Keep track of the max ID index (which will be used to generate new objects). --->
			<cfset VARIABLES.Instance.IDIndex = Max( VARIABLES.Instance.IDIndex, LOCAL.Group.ID ) />
			
		</cfloop>
		
		
		<!--- Re-order them for proper date. --->
		<cfquery name="VARIABLES.Instance.Groups" dbtype="query">
			SELECT
				*
			FROM
				VARIABLES.Instance.Groups
			ORDER BY
				Name ASC		
		</cfquery>
		
		
		<!--- Return out. --->
		<cfreturn />
	</cffunction>
	
	
	<cffunction 
		name="New" 
		access="public" 
		returntype="struct" 
		output="false" 
		hint="I create a new Group object.">
	
		<!--- Define the local scope. --->
		<cfset var LOCAL = {} />
		
		<!--- Create a default Group struct. --->
		<cfset LOCAL.Group = {
			ID = (++VARIABLES.Instance.IDIndex),
			Name = "",
			IsRemovedFromGeneralTweets = false,
			IncludePatterns = "",
			ExcludePatterns = "",
			DateCreated = Now()
			} />
			
		<!--- Return the new group. --->
		<cfreturn LOCAL.Group />
	</cffunction>
	
	
	<cffunction 
		name="PersistGroupData" 
		access="private" 
		returntype="void" 
		output="false" 
		hint="I persist the group data file to disk.">
		
		<!--- Define the local scope. --->
		<cfset var LOCAL = {} />
		
		<!--- Write the group JSON file. --->
		<cffile
			action="write"
			file="#VARIABLES.DataFile#"
			output="#SerializeJSON( VARIABLES.QueryToArray( VARIABLES.Instance.Groups ) )#"
			/>
		
		<!--- Return out. --->
		<cfreturn />
	</cffunction>
	
	
	<cffunction 
		name="Save" 
		access="public" 
		returntype="struct" 
		output="false" 
		hint="I save the given Group object.">
	
		<!--- Define aruments. --->	
		<cfargument 
			name="Group" 
			type="struct" 
			required="true" 
			hint="I am Group object returned from New()." 
			/>
			
		<!--- Remove any internal record temporarily. --->
		<cfquery name="VARIABLES.Instance.Groups" dbtype="query">
			SELECT
				*
			FROM
				VARIABLES.Instance.Groups
			WHERE
				ID != <cfqueryparam value="#ARGUMENTS.Group.ID#" cfsqltype="cf_sql_integer" />		
		</cfquery>
		
		<!--- 
			Now that we have removed the old row, we can re-insert it with the 
			new and / or updated information.
		--->
			
		<!--- Add a query row. --->
		<cfset QueryAddRow( VARIABLES.Instance.Groups ) />
		
		<!--- Set column values. --->
		<cfset VARIABLES.Instance.Groups[ "ID" ][ VARIABLES.Instance.Groups.RecordCount ] = JavaCast( "int", ARGUMENTS.Group.ID ) />
		<cfset VARIABLES.Instance.Groups[ "Name" ][ VARIABLES.Instance.Groups.RecordCount ] = JavaCast( "string", ARGUMENTS.Group.Name ) />
		<cfset VARIABLES.Instance.Groups[ "IsRemovedFromGeneralTweets" ][ VARIABLES.Instance.Groups.RecordCount ] = JavaCast( "boolean", ARGUMENTS.Group.IsRemovedFromGeneralTweets ) />
		<cfset VARIABLES.Instance.Groups[ "IncludePatterns" ][ VARIABLES.Instance.Groups.RecordCount ] = JavaCast( "string", ARGUMENTS.Group.IncludePatterns ) />
		<cfset VARIABLES.Instance.Groups[ "ExcludePatterns" ][ VARIABLES.Instance.Groups.RecordCount ] = JavaCast( "string", ARGUMENTS.Group.ExcludePatterns ) />
		<cfset VARIABLES.Instance.Groups[ "DateCreated" ][ VARIABLES.Instance.Groups.RecordCount ] = ARGUMENTS.Group.DateCreated />
	
		<!--- Update the file data. --->
		<cfset VARIABLES.PersistGroupData() />
		
		<!--- Return group. --->
		<cfreturn ARGUMENTS.Group />
	</cffunction>
	
</cfcomponent>