
<cfcomponent
	extends="BaseService"
	output="false"
	hint="I provide data logging functions.">

	
	<cffunction 
		name="Init" 
		access="public" 
		returntype="any" 
		output="false" 
		hint="I return an intialized object.">
		
		<!--- Define arguments. --->
		<cfargument 
			name="Factory" 
			type="any" 
			required="true" 
			hint="I am the applications object factory."
			/>
		
		<cfargument 
			name="LoggingFolder" 
			type="string" 
			required="true" 
			hint="I am the logging folder for this application."
			/>
			
		<!--- Store the factory. --->
		<cfset VARIABLES.Factory = ARGUMENTS.Factory />
		
		<!--- Store the data folder. --->
		<cfset VARIABLES.LoggingFolder = ARGUMENTS.LoggingFolder />
				
		<!--- Return This reference. --->
		<cfreturn THIS />
	</cffunction>
	
	
	<cffunction 
		name="LogData" 
		access="public" 
		returntype="void" 
		output="false"
		hint="I log the given data to the logging folder.">
	
		<!--- Define arguments. --->
		<cfargument 
			name="Type" 
			type="string" 
			required="true" 
			hint="I am the type of data being logged." 
			/>
			
		<cfargument 
			name="Data" 
			type="any" 
			required="true" 
			hint="I am the data being logged."
			/>
			
		<cfargument 
			name="Suffix" 
			type="string" 
			required="false" 
			default="" 
			hint="I am the suffix to add to the log file as a convenience."
			/>
		
		<!--- Define the local scope. --->
		<cfset var LOCAL = {} />
		
		
		<!--- 
			Increase the page load time if there was an error. This is just 
			precautionary in case the CFCatch was handling a time-out issue.
		--->
		<cfsetting requesttimeout="60" />
		
		
		<!--- Create the HTML variable. --->
		<cfsavecontent variable="LOCAL.Output">
			<cfoutput>
				
				<h1>
					Logging - #ARGUMENTS.Type#
				</h1>
				
				<p>
					Logged on 
					#DateFormat( Now(), "mmm d, yyyy" )# at
					#TimeFormat( Now(), "h:mm TT" )#
				</p>
				
				<cfdump 
					var="#ARGUMENTS.Data#"
					label="Logged Data"
					/>
					
			</cfoutput>
		</cfsavecontent>
		
		
		<!--- 
			Check to see if there is a suffix. If so, then prepend the 
			underscore for output.
		--->
		<cfif Len( ARGUMENTS.Suffix )>
			
			<cfset ARGUMENTS.Suffix = ("_" & ARGUMENTS.Suffix) />
		
		</cfif>
		
		
		<!--- Create a file name. --->
		<cfset LOCAL.FileName = (
			"log_" &
			DateFormat( Now(), "yyyy_mm_yy" ) & 
			"_" &
			TimeFormat( Now(), "HH_mm_l" ) &
			ARGUMENTS.Suffix &
			".htm"			
			) />
		
		<!--- Write to the log directory. --->
		<cffile
			action="write"
			file="#VARIABLES.LoggingFolder##LOCAL.FileName#"
			output="#LOCAL.Output#"
			/>
		
		<!--- Return out. --->
		<cfreturn />
	</cffunction>
	
</cfcomponent>