
<cfcomponent
	extends="BaseService"
	output="false"
	hint="I provide service methods for Status objects.">

	
	<cffunction 
		name="Init" 
		access="public" 
		returntype="any" 
		output="false" 
		hint="I return an intialized object.">
		
		<!--- Define arguments. --->
		<cfargument 
			name="Factory" 
			type="any" 
			required="true" 
			hint="I am the applications object factory."
			/>
		
		<cfargument 
			name="DataFolder" 
			type="string" 
			required="true" 
			hint="I am the data folder for this application."
			/>
		
		<!--- Store the factory. --->
		<cfset VARIABLES.Factory = ARGUMENTS.Factory />
		
		<!--- Store the data folder. --->
		<cfset VARIABLES.DataFolder = ARGUMENTS.DataFolder />
		
		<!--- Store the file path. --->
		<cfset VARIABLES.DataFile = (VARIABLES.DataFolder & "statuses.json") />
		
		<!--- Set up instance variables. --->
		<cfset VARIABLES.Instance = {
			Statuses = QueryNew(
				"ID,TwitterID,Status,IsGeneral,IsReply,IsDirect,IsSnoozed,FriendID,GroupID,DateCreated",
				"cf_sql_integer,cf_sql_integer,cf_sql_varchar,cf_sql_bit,cf_sql_bit,cf_sql_bit,cf_sql_bit,cf_sql_integer,cf_sql_integer,cf_sql_timestamp"
				),
			IDIndex = 0
			} />
			
		<!--- Load the status data from file. --->	
		<cfset VARIABLES.LoadStatusData() />
		
		<!--- Return This reference. --->
		<cfreturn THIS />
	</cffunction>
	
	
	<cffunction 
		name="Delete" 
		access="public" 
		returntype="struct" 
		output="false" 
		hint="I delete the given Status at the given ID.">
		
		<!--- Define arguments. --->
		<cfargument 
			name="ID" 
			type="numeric" 
			required="true" 
			hint="I am the internal ID of the given status."
			/>
			
		<!--- Define the local scope. --->
		<cfset var LOCAL = {} />
		
		<!--- Get the given status. --->
		<cfset LOCAL.Status = THIS.Get( ARGUMENTS.ID ) />
				
		<!--- Delete the given record. --->
		<cfquery name="VARIABLES.Instance.Statuses" dbtype="query">
			SELECT 
				* 
			FROM
				VARIABLES.Instance.Statuses
			WHERE
				ID != <cfqueryparam value="#ARGUMENTS.ID#" cfsqltype="cf_sql_integer" />
			ORDER BY
				TwitterID DESC
		</cfquery>
		
		<!--- Persist the status data. --->
		<cfset VARIABLES.PersistStatusData() />
		
		<!--- Return the Status. --->
		<cfreturn LOCAL.Status />
	</cffunction>
	
	
	<cffunction 
		name="DeleteByFriend"
		access="public" 
		returntype="void" 
		output="false" 
		hint="I delete all the statuses in a given friend.">
	
		<!--- Define arguments. --->	
		<cfargument 
			name="Friend" 
			type="struct" 
			required="true" 
			hint="The friend we are deleting statuses for."
			/>
		
		<!--- Delete status associated to the given friend. --->
		<cfquery name="VARIABLES.Instance.Statuses" dbtype="query">
			SELECT 
				* 
			FROM
				VARIABLES.Instance.Statuses
			WHERE
				FriendID != <cfqueryparam value="#ARGUMENTS.Friend.ID#" cfsqltype="cf_sql_integer" />
			ORDER BY
				TwitterID DESC
		</cfquery>
		
		<!--- Persist the status data. --->
		<cfset VARIABLES.PersistStatusData() />
		
		<!--- Return out. --->
		<cfreturn />
	</cffunction>
	
	
	<cffunction 
		name="DeleteByGroup"
		access="public" 
		returntype="void" 
		output="false" 
		hint="I delete all the statuses in a given group.">
	
		<!--- Define arguments. --->	
		<cfargument 
			name="Group" 
			type="struct" 
			required="true" 
			hint="The group we are deleting statuses for."
			/>
		
		<!--- Delete status associated to the given group. --->
		<cfquery name="VARIABLES.Instance.Statuses" dbtype="query">
			SELECT 
				* 
			FROM
				VARIABLES.Instance.Statuses
			WHERE
				GroupID != <cfqueryparam value="#ARGUMENTS.Group.ID#" cfsqltype="cf_sql_integer" />
			ORDER BY
				TwitterID DESC
		</cfquery>
		
		<!--- Persist the status data. --->
		<cfset VARIABLES.PersistStatusData() />
		
		<!--- Return out. --->
		<cfreturn />
	</cffunction>
	
	
	<cffunction 
		name="Get" 
		access="public" 
		returntype="struct" 
		output="false" 
		hint="I return the given Status at the given ID.">
		
		<!--- Define arguments. --->
		<cfargument 
			name="ID" 
			type="numeric" 
			required="true" 
			hint="I am the internal ID of the given status."
			/>
			
		<!--- Define the local scope. --->
		<cfset var LOCAL = {} />
		
		<!--- Query for the current row. --->
		<cfquery name="LOCAL.StatusData" dbtype="query">
			SELECT
				*
			FROM
				VARIABLES.Instance.Statuses
			WHERE
				ID = <cfqueryparam value="#ARGUMENTS.ID#" cfsqltype="cf_sql_integer" />
		</cfquery>
		
		<!--- Create the intermediar status sturct. --->
		<cfset LOCAL.Status = VARIABLES.QueryRowToStruct( LOCAL.StatusData ) />
		
		<!--- Return the Status. --->
		<cfreturn LOCAL.Status />
	</cffunction>
	
	
	<cffunction 
		name="GetAll" 
		access="public" 
		returntype="array" 
		output="false" 
		hint="I return the collection of Statuses.">
			
		<!--- Return the Status. --->
		<cfreturn VARIABLES.QueryToArray( VARIABLES.Instance.Statuses ) />
	</cffunction>
	
	
	<cffunction 
		name="GetMostRecentTwitterID" 
		access="public" 
		returntype="numeric" 
		output="false" 
		hint="I return the most recent status ID (Twitter ID).">
		
		<!--- Define arguments. --->
		<cfargument 
			name="IsDirect" 
			type="boolean" 
			required="false" 
			default="false" 
			hint="I flag whether or not to get ID for direct messages." 
			/>
		
		<!--- Define the local scope. --->
		<cfset var LOCAL = {} />
		
		<!--- Query for the largest twitter ID --->
		<cfquery name="LOCAL.MaxTwitter" dbtype="query">
			SELECT
				MAX( TwitterID ) AS ID
			FROM
				VARIABLES.Instance.Statuses
			WHERE
				<cfif ARGUMENTS.IsDirect>
					IsDirect = 1
				<cfelse>
					IsDirect = 0
				</cfif>
		</cfquery>
		
		<!--- Return the max Twitter ID. --->
		<cfreturn Val( LOCAL.MaxTwitter.ID ) />
	</cffunction>
	
	
	<cffunction 
		name="GetRecent" 
		access="public" 
		returntype="array" 
		output="false" 
		hint="I return the array of most recent statuses created after the given ID.">
		
		<!--- Define arguments. --->
		<cfargument 
			name="ID" 
			type="numeric" 
			required="false" 
			default="0"
			hint="I am ID of the cutoff." 
			/>
		
		<!--- Define the local scope. --->
		<cfset var LOCAL = {} />
		
		<!--- Query for the recent twitters. --->
		<cfquery name="LOCAL.RecentStatuses" dbtype="query">
			SELECT
				*
			FROM
				VARIABLES.Instance.Statuses
			WHERE
				ID > <cfqueryparam value="#ARGUMENTS.ID#" cfsqltype="cf_sql_integer" />
				
			<!--- When getting recent tweets, order by the oldest one's first. --->
			ORDER BY
				TwitterID ASC
		</cfquery>
		
		<!--- Return recent statuses. --->
		<cfreturn VARIABLES.QueryToArray( LOCAL.RecentStatuses ) />
	</cffunction>
	
	
	<cffunction 
		name="LoadStatusData" 
		access="private" 
		returntype="void" 
		output="false" 
		hint="I load the status data file from disk.">
		
		<!--- Define the local scope. --->
		<cfset var LOCAL = {} />
		
		<!--- Try to load the file data - it might not exist yet. --->
		<cftry>
		
			<!--- Load the status JSON file. --->
			<cffile
				action="read"
				file="#VARIABLES.DataFile#"
				variable="LOCAL.StatusData"
				/>
			
			<!--- Deserialize the JSON and store internally as an array. --->
			<cfset LOCAL.Statuses = DeserializeJSON( LOCAL.StatusData ) />
			
			<!--- Catch any errors. --->
			<cfcatch>
				
				<!--- 
					The file did not exist or there was an error in the JSON data. 
					Just create a default structure as our disaster recovery plan.
				--->
				<cfset LOCAL.Statuses = [] />
				
			</cfcatch>
		</cftry>
		
		
		<!--- 
			Now that we have the statuses array, we need to convert that to out 
			internal query structure.
		--->
		<cfloop
			index="LOCAL.Status"
			array="#LOCAL.Statuses#">
				
			<!--- Add a query row. --->
			<cfset QueryAddRow( VARIABLES.Instance.Statuses ) />
			
			<!--- Set column values. --->
			<cfset VARIABLES.Instance.Statuses[ "ID" ][ VARIABLES.Instance.Statuses.RecordCount ] = JavaCast( "int", LOCAL.Status.ID ) />
			<cfset VARIABLES.Instance.Statuses[ "TwitterID" ][ VARIABLES.Instance.Statuses.RecordCount ] = JavaCast( "int", LOCAL.Status.TwitterID ) />
			<cfset VARIABLES.Instance.Statuses[ "Status" ][ VARIABLES.Instance.Statuses.RecordCount ] = JavaCast( "string", LOCAL.Status.Status ) />
			<cfset VARIABLES.Instance.Statuses[ "IsGeneral" ][ VARIABLES.Instance.Statuses.RecordCount ] = JavaCast( "boolean", LOCAL.Status.IsGeneral ) />
			<cfset VARIABLES.Instance.Statuses[ "IsReply" ][ VARIABLES.Instance.Statuses.RecordCount ] = JavaCast( "boolean", LOCAL.Status.IsReply ) />
			<cfset VARIABLES.Instance.Statuses[ "IsDirect" ][ VARIABLES.Instance.Statuses.RecordCount ] = JavaCast( "boolean", LOCAL.Status.IsDirect ) />
			<cfset VARIABLES.Instance.Statuses[ "IsSnoozed" ][ VARIABLES.Instance.Statuses.RecordCount ] = JavaCast( "boolean", LOCAL.Status.IsSnoozed ) />
			<cfset VARIABLES.Instance.Statuses[ "FriendID" ][ VARIABLES.Instance.Statuses.RecordCount ] = JavaCast( "int", LOCAL.Status.FriendID ) />
			<cfset VARIABLES.Instance.Statuses[ "GroupID" ][ VARIABLES.Instance.Statuses.RecordCount ] = JavaCast( "int", LOCAL.Status.GroupID ) />
			<cfset VARIABLES.Instance.Statuses[ "DateCreated" ][ VARIABLES.Instance.Statuses.RecordCount ] = ParseDateTime( LOCAL.Status.DateCreated ) />
					
			<!--- Keep track of the max ID index (which will be used to generate new objects). --->
			<cfset VARIABLES.Instance.IDIndex = Max( VARIABLES.Instance.IDIndex, LOCAL.Status.ID ) />
			
		</cfloop>
		
		
		<!--- Re-order them for proper date. --->
		<cfquery name="VARIABLES.Instance.Statuses" dbtype="query">
			SELECT
				*
			FROM
				VARIABLES.Instance.Statuses
			ORDER BY
				TwitterID DESC
		</cfquery>
		
		
		<!--- Return out. --->
		<cfreturn />
	</cffunction>
	
	
	<cffunction 
		name="New" 
		access="public" 
		returntype="struct" 
		output="false" 
		hint="I create a new Status object.">
	
		<!--- Define the local scope. --->
		<cfset var LOCAL = {} />
		
		<!--- Create a default Status struct. --->
		<cfset LOCAL.Status = {
			ID = (++VARIABLES.Instance.IDIndex),
			TwitterID = 0,
			Status = "",
			IsGeneral = true,
			IsReply = false,
			IsDirect = false,
			IsSnoozed = false,
			FriendID = 0,
			GroupID = 0,
			DateCreated = Now()
			} />
			
		<!--- Return the new status. --->
		<cfreturn LOCAL.Status />
	</cffunction>
	
	
	<cffunction 
		name="PersistStatusData" 
		access="private" 
		returntype="void" 
		output="false" 
		hint="I persist the status data file to disk.">
		
		<!--- Define the local scope. --->
		<cfset var LOCAL = {} />
		
		<!--- 
			When we persiste, we only want to store the most recent 200 messages. 
			Anything after that is someone pointless (in terms of usability) -
			after all, this is a CHAT program really.
			
			When sorting this, sort by internal ID rather than TwitterID since
			the TwitterID's are too eratic. 
		--->
		<cfquery name="VARIABLES.Instance.Statuses" dbtype="query" maxrows="200">
			SELECT
				*
			FROM
				VARIABLES.Instance.Statuses
			ORDER BY
				ID DESC
		</cfquery>
		
		
		<!--- Write the status JSON file. --->
		<cffile
			action="write"
			file="#VARIABLES.DataFile#"
			output="#SerializeJSON( VARIABLES.QueryToArray( VARIABLES.Instance.Statuses ) )#"
			/>
			
			
		<!--- Now that we saved the data, re-sort it. --->
		<cfquery name="VARIABLES.Instance.Statuses" dbtype="query" maxrows="200">
			SELECT
				*
			FROM
				VARIABLES.Instance.Statuses
			ORDER BY
				TwitterID DESC
		</cfquery>
		
		<!--- Return out. --->
		<cfreturn />
	</cffunction>
	
	
	<cffunction 
		name="Save" 
		access="public" 
		returntype="struct" 
		output="false" 
		hint="I save the given Status object.">
	
		<!--- Define aruments. --->	
		<cfargument 
			name="Status" 
			type="struct" 
			required="true" 
			hint="I am Status object returned from New()." 
			/>
			
		<!--- Remove any internal record temporarily. --->
		<cfquery name="VARIABLES.Instance.Statuses" dbtype="query">
			SELECT
				*
			FROM
				VARIABLES.Instance.Statuses
			WHERE
				ID != <cfqueryparam value="#ARGUMENTS.Status.ID#" cfsqltype="cf_sql_integer" />		
		</cfquery>
		
		<!--- 
			Now that we have removed the old row, we can re-insert it with the 
			new and / or updated information.
		--->
			
		<!--- Add a query row. --->
		<cfset QueryAddRow( VARIABLES.Instance.Statuses ) />
		
		<!--- Set column values. --->
		<cfset VARIABLES.Instance.Statuses[ "ID" ][ VARIABLES.Instance.Statuses.RecordCount ] = JavaCast( "int", ARGUMENTS.Status.ID ) />
		<cfset VARIABLES.Instance.Statuses[ "TwitterID" ][ VARIABLES.Instance.Statuses.RecordCount ] = JavaCast( "int", ARGUMENTS.Status.TwitterID ) />
		<cfset VARIABLES.Instance.Statuses[ "Status" ][ VARIABLES.Instance.Statuses.RecordCount ] = JavaCast( "string", ARGUMENTS.Status.Status ) />
		<cfset VARIABLES.Instance.Statuses[ "IsGeneral" ][ VARIABLES.Instance.Statuses.RecordCount ] = JavaCast( "boolean", ARGUMENTS.Status.IsGeneral ) />
		<cfset VARIABLES.Instance.Statuses[ "IsReply" ][ VARIABLES.Instance.Statuses.RecordCount ] = JavaCast( "boolean", ARGUMENTS.Status.IsReply ) />
		<cfset VARIABLES.Instance.Statuses[ "IsDirect" ][ VARIABLES.Instance.Statuses.RecordCount ] = JavaCast( "boolean", ARGUMENTS.Status.IsDirect ) />
		<cfset VARIABLES.Instance.Statuses[ "IsSnoozed" ][ VARIABLES.Instance.Statuses.RecordCount ] = JavaCast( "boolean", ARGUMENTS.Status.IsSnoozed ) />
		<cfset VARIABLES.Instance.Statuses[ "FriendID" ][ VARIABLES.Instance.Statuses.RecordCount ] = JavaCast( "int", ARGUMENTS.Status.FriendID ) />
		<cfset VARIABLES.Instance.Statuses[ "GroupID" ][ VARIABLES.Instance.Statuses.RecordCount ] = JavaCast( "int", ARGUMENTS.Status.GroupID ) />
		<cfset VARIABLES.Instance.Statuses[ "DateCreated" ][ VARIABLES.Instance.Statuses.RecordCount ] = ParseDateTime( ARGUMENTS.Status.DateCreated ) />
		
		<!--- Update the file data. --->
		<cfset VARIABLES.PersistStatusData() />
		
		<!--- Return status. --->
		<cfreturn ARGUMENTS.Status />
	</cffunction>

</cfcomponent>