
<cfcomponent
	extends="BaseService"
	output="false"
	hint="I provide service functions for Twitter.">

	
	<cffunction 
		name="Init" 
		access="public" 
		returntype="any" 
		output="false" 
		hint="I return an intialized object.">
		
		<!--- Define arguments. --->
		<cfargument 
			name="Factory" 
			type="any" 
			required="true" 
			hint="I am the applications object factory."
			/>
		
		<cfargument 
			name="Username" 
			type="string" 
			required="true" 
			hint="I am the twitter username."
			/>
			
		<cfargument 
			name="Password" 
			type="string" 
			required="true" 
			hint="I am the twitter password."
			/>
		
		<!--- Store the factory. --->
		<cfset VARIABLES.Factory = ARGUMENTS.Factory />
		
		<!--- Set up instance variables. --->
		<cfset VARIABLES.Instance = {
			Username = ARGUMENTS.Username,
			Password = ARGUMENTS.Password,
			DateTwitterLastPinged = "",
			TwitterPingCount = 0,
			APILimit = 100
			} />
			
		<!--- Return This reference. --->
		<cfreturn THIS />
	</cffunction>
	
	
	<cffunction
		name="ParseStatusText"
		access="public"
		returntype="string"
		output="false"
		hint="I parse the HTML of a twitter status text.">
		
		<!--- Define arguments. --->
		<cfargument 
			name="Text" 
			type="string" 
			required="true" 
			hint="I am the status text to parse." 
			/>
		
		<!--- Define the local scope. --->
		<cfset var LOCAL = {} />
		
		<!--- Replace links with link tags. --->
		<cfset ARGUMENTS.Text = ARGUMENTS.Text.ReplaceAll(
			JavaCast( "string", "(?i)(https?://[\w\.\-_\/\\##\?\+\=]+[\w\/\\##\?\+\=])" ),
			JavaCast( "string", "<a href=""$1"" target=""_blank"">$1</a>" )
			) />
		
		<!--- Replace twitter screen names with twitter links. --->
		<cfset ARGUMENTS.Text = ARGUMENTS.Text.ReplaceAll(
			JavaCast( "string", "@(\w+)\b" ),
			JavaCast( "string", "@<a href=""http://www.twitter.com/$1"" target=""_blank"">$1</a>" )
			) />
		
		<!--- Return the updated link. --->
		<cfreturn ARGUMENTS.Text />		
	</cffunction>
	
		
	<cffunction 
		name="SubmitStatus" 
		access="public" 
		returntype="boolean"
		output="false" 
		hint="I submit a status message to Twitter.">
	
		<!--- Define arguments. --->
		<cfargument 
			name="Status"
			type="string" 
			required="true" 
			hint="I am the status being submitted to Twitter."
			/>
		
		
		<!--- Define the local scope. --->
		<cfset var LOCAL = {} />
		
		<!--- Set the default return flag. --->
		<cfset LOCAL.Success = true />
		
		
		<cftry>
		
			<!--- Clean the status. --->
			<cfset LOCAL.Status = Trim( ARGUMENTS.Status ) />
			
			
			<!--- Check to see if the status is a direct message. --->
			<cfif REFindNoCase( "^D ", LOCAL.Status )>
			
				<!--- Direct message. --->
				
				<!--- Strip out the "D" flag. --->
				<cfset LOCAL.Status = LTrim( REReplaceNoCase( LOCAL.Status, "^D ", "", "one" ) ) />
				
				<!--- Get the target user. --->
				<cfset LOCAL.Friend = ListGetAt( LOCAL.Status, 1, " " ) />
				
				<!--- Strip out friend from status. --->
				<cfset LOCAL.Status = LTrim( REReplaceNoCase( LOCAL.Status, "^[^\s]+", "", "one" ) ) />
						
						
				<!--- Build the parameters to pass along. --->
				<cfset LOCAL.Parameters = [] />
				
				
				<!--- Create count parameter. --->
				<cfset LOCAL.UserParameter = {
					Name = "user",
					Value = LOCAL.Friend
					} />
					
				<!--- Add to parameter collection for API request. --->
				<cfset ArrayAppend(
					LOCAL.Parameters,
					LOCAL.UserParameter
					) />
				
				
				<!--- Create text parameter. --->
				<cfset LOCAL.TextParameter = {
					Name = "text",
					Value = LOCAL.Status
					} />
					
				<!--- Add to parameter collection for API request. --->
				<cfset ArrayAppend(
					LOCAL.Parameters,
					LOCAL.TextParameter
					) />
						
						
				<!--- Make sure that the status has any length. --->
				<cfif NOT Len( LOCAL.TextParameter.Value )>
				
					<!--- Throw length exception. --->
					<cfthrow
						type="KinkyTwits.TwitterService.SubmitStatus.InvalidValue"
						message="STATUS does not have a valid length."
						detail="STATUS is empty. Status must be between 1 and 140 characters."
						/>
				
				</cfif>
				
				<!--- Send the request. --->
				<cfset LOCAL.TwitterResponse = THIS.SubmitTwitterAPIRequest(
					RequestMethod = "post",
					Resource = "direct_messages/new",
					ID = "",
					Parameters = LOCAL.Parameters
					) />
			
			<cfelse>
				
				<!--- Standard message. --->
				
				<!--- 
					Build the post parameters. Make sure that the post is not 
					more than 160 characters. 
				--->
				
				<!--- Build the parameters to pass along. --->
				<cfset LOCAL.Parameters = [] />
				
				
				<!--- Create status parameter. --->
				<cfset LOCAL.StatusParameter = {
					Name = "status",
					Value = LOCAL.Status
					} />
					
				<!--- Add to parameter collection for API request. --->
				<cfset ArrayAppend(
					LOCAL.Parameters,
					LOCAL.StatusParameter
					) />
				
				
				<!--- Make sure that the status has any length. --->
				<cfif NOT Len( LOCAL.StatusParameter.Value )>
				
					<!--- Throw length exception. --->
					<cfthrow
						type="KinkyTwits.TwitterService.SubmitStatus.InvalidValue"
						message="STATUS does not have a valid length."
						detail="STATUS is empty. Status must be between 1 and 160 characters."
						/>
						
				<cfelseif REFind( "^(\s*@[\w]+\s*)+$", LOCAL.StatusParameter.Value )>
				
					<!--- Throw content exception. --->
					<cfthrow
						type="KinkyTwits.TwitterService.SubmitStatus.InvalidValue"
						message="STATUS does not have valid content."
						detail="STATUS is does not have valid content. Status must contain more than just a target user."
						/>
				
				</cfif>
				
				<!--- Send the request. --->
				<cfset LOCAL.TwitterResponse = THIS.SubmitTwitterAPIRequest(
					RequestMethod = "post",
					Resource = "statuses/update",
					ID = "",
					Parameters = LOCAL.Parameters
					) />
			
			</cfif>
				
					
			<!--- Catch any exceptions. --->
			<cfcatch>
				
				<!--- Log error. --->
				<cfset VARIABLES.Factory.Get( "LoggingService" ).LogData(
					"SubmitStatus() Error",
					CFCATCH,
					"error"
					) />
					
				<!--- Set success flag for failure. --->
				<cfset LOCAL.Success = false />
					
			</cfcatch>			
		</cftry>	
		
		<!--- Return success flag. --->
		<cfreturn LOCAL.Success />
	</cffunction>
	
	
	<cffunction 
		name="SubmitTwitterAPIRequest" 
		access="public" 
		returntype="any" 
		output="true" 
		hint="I submit the raw Twitter API request.">
		
		<!--- Define arguments. --->
		<cfargument 
			name="RequestMethod" 
			type="string" 
			required="true" 
			hint="I am the type of method to use when making the request - GET or POST." 
			/>
			
		<cfargument 
			name="Resource" 
			type="string" 
			required="true" 
			hint="I am the twitter resource upon which we are acting. Ex. statuses, users."
			/>
		
		<cfargument 
			name="ID" 
			type="string" 
			required="false" 
			default="" 
			hint="I am the ID of the item upon which we are acting." 
			/>
		
		<cfargument 
			name="Parameters" 
			type="array" 
			required="false" 
			default="#ArrayNew( 1 )#" 
			hint="I am an array of additional name-value struct parameters that can be posted." 
			/>
		
		<!--- Define the local scope. --->
		<cfset var LOCAL = {} />
		
		<!--- Set the request format. --->
		<cfset LOCAL.ReturnFormat = "json" />
		
		
		<!--- 
			Start building the request path. If we have an ID, we have to 
			include that in the requested URL.
		--->
		<cfif Len( ARGUMENTS.ID )>
		
			<!--- Build ID-based URL. --->
			<cfset LOCAL.URL = "http://www.twitter.com/#ARGUMENTS.Resource#/#ARGUMENTS.ID#.#LOCAL.ReturnFormat#" />
		
		<cfelse>
		
			<!--- Build reource-only URL. --->
			<cfset LOCAL.URL = "http://www.twitter.com/#ARGUMENTS.Resource#.#LOCAL.ReturnFormat#" />
		
		</cfif>
		
	
		<!--- Make the twitter request. --->
		<cfhttp
			method="#ARGUMENTS.RequestMethod#"
			url="#LOCAL.URL#"
			useragent="KinkyTwitsTwitterBot"
			username="#VARIABLES.Instance.Username#"
			password="#VARIABLES.Instance.Password#"
			result="LOCAL.TwitterRequest">
		
			<!--- 
				Loop over any additional paramters that need to be passed along. If this 
				is a GET request, we need to add URL params. If this is a POST request, 
				we need to send FORM values.
			--->
			<cfif (ARGUMENTS.RequestMethod EQ "Get")>
			
				<!--- Make URL parameters for GET method. --->
				<cfloop
					index="LOCAL.Parameter"
					array="#ARGUMENTS.Parameters#">
					
					<cfhttpparam
						type="url"
						name="#LCase( LOCAL.Parameter.Name )#"
						value="#LOCAL.Parameter.Value#"
						/>
					
				</cfloop>			
			
			<cfelse>
			
				<!--- Make FormField parameters for POST method. --->
				<cfloop
					index="LOCAL.Parameter"
					array="#ARGUMENTS.Parameters#">
					
					<cfhttpparam
						type="formfield"
						name="#LCase( LOCAL.Parameter.Name )#"
						value="#LOCAL.Parameter.Value#"
						/>
					
				</cfloop>
			
			</cfif>	
			
		</cfhttp>
		
		
		<!---
		<!--- Log response. --->
		<cfset LOCAL.Settings = {
			Url = LOCAL.URL,
			PingCount = VARIABLES.Instance.TwitterPingCount,
			Arguments = ARGUMENTS,
			Response = ToString( LOCAL.TwitterRequest.FileContent )
			} />
		
		<cfset VARIABLES.Factory.Get( "LoggingService" ).LogData(
			"Twitter CFHTTP Response", 
			LOCAL.Settings,
			"response"			
			) />
		--->
		
		<!--- 
			Check to see if the status was 200. If not, let's raise an exception 
			and the let the calling method handle the failure.
		--->
		<cfif NOT FindNoCase( "200", LOCAL.TwitterRequest.StatusCode )>
		
			<!--- Raise request exception. --->
			<cfthrow
				type="KinkyTwits.TwitterService.SubmitTwitterAPIRequest.TwitterRequestFailure.#REReplace( LOCAL.TwitterRequest.StatusCode, '[^\d]+', '', 'all' )#"
				message="Twitter API request failed."
				detail="Twitter request to, #UCase( LOCAL.URL )#, returned a #UCase( LOCAL.TwitterRequest.StatusCode )# status code."
				/>
		
		</cfif>
		
		
		<!--- 
			ASSERT: If we have made it this far, then the twitter request 
			was successful.
		--->
		
		
		<!--- Convert the response to something usable. --->
		<cfset LOCAL.Response = {
			RawData = ToString( LOCAL.TwitterRequest.FileContent ),
			StatusCode = LOCAL.TwitterRequest.StatusCode,
			Header = LOCAL.TwitterRequest.ResponseHeader
			} />
			
			
		<!--- Convert the binary response to a ColdFusion object. --->
		<cftry>
		
			<!--- Deserialize the JSON. --->
			<cfset LOCAL.Response.Data = DeserializeJSON( LOCAL.Response.RawData ) />
			
			<!--- Catch any errors. --->
			<cfcatch>
			
				<!--- Log error. --->
				<cfset VARIABLES.Factory.Get( "LoggingService" ).LogData(
					"Twitter JSON Data Error", 
					CFCATCH,
					"json_error"			
					) />
					
				<!--- If there were any errors, just set to empty struct. --->
				<cfset LOCAL.Response.Data = {} />
			
			</cfcatch>			
		</cftry>
		
		
		<!--- Return Response. --->
		<cfreturn LOCAL.Response />
	</cffunction>
	
	
	<cffunction 
		name="UpdateDirectStatuses" 
		access="public" 
		returntype="void" 
		output="false" 
		hint="I get the latest DIRECT Twitter status list and update the local database.">
		
		<!--- Define the local scope. --->
		<cfset var LOCAL = {} />
		
		<!--- Get the last ID. --->
		<cfset LOCAL.LastTwitterID = VARIABLES.Factory.Get( "StatusService" ).GetMostRecentTwitterID(
			IsDirect = true
			) />
		
		<!--- Build the parameters to pass along. --->
		<cfset LOCAL.Parameters = [] />
		
		<!--- Check for twitter ID. --->
		<cfif LOCAL.LastTwitterID>
		
			<!--- Define parameter. --->
			<cfset LOCAL.SinceIDParameter = {
				Name = "since_id",
				Value = LOCAL.LastTwitterID
				} />
				
			<!--- Add to parameter collection for API request. --->
			<cfset ArrayAppend(
				LOCAL.Parameters,
				LOCAL.SinceIDParameter
				) />
		
		</cfif>
		
		
		<!--- Create count parameter. --->
		<cfset LOCAL.CountParameter = {
			Name = "count",
			Value = 200
			} />
			
		<!--- Add to parameter collection for API request. --->
		<cfset ArrayAppend(
			LOCAL.Parameters,
			LOCAL.CountParameter
			) />
		
		
		<!--- Make sure this doesn't break in any way. --->
		<cftry>
			
			<!--- Send the request. --->
			<cfset LOCAL.TwitterResponse = THIS.SubmitTwitterAPIRequest(
				RequestMethod = "get",
				Resource = "direct_messages",
				ID = "",
				Parameters = LOCAL.Parameters
				) />
		
							
			<!--- Loop over the returned statuses. --->
			<cfloop 
				index="LOCAL.StatusData"
				array="#LOCAL.TwitterResponse.Data#">
						
						
				<!--- Format date properly. Sun Oct 05 21:50:35 +0000 2008. --->
				<cfset LOCAL.DateStatusCreated = REReplaceNoCase(
					LOCAL.StatusData.created_at,
					"(\w+) (\w+ \d+) (\d+:\d+:\d+) \+\d+ (\d+)",
					"\2 \4 \3",
					"one"								
					) />
							
				<!--- Store status date back in. --->
				<cfset LOCAL.StatusData.created_at = ParseDateTime( LOCAL.DateStatusCreated ) />
				
				
				<!--- Try to get this friend based on the twitter ID. --->
				<cftry>
					<cfset LOCAL.StatusFriend = VARIABLES.Factory.Get( "FriendService" ).GetByTwitterID( LOCAL.StatusData.User.ID ) />
				
					<!--- Catch error if friend does not exist. --->
					<cfcatch>
						
						<!--- Create new friend. --->
						<cfset LOCAL.StatusFriend = VARIABLES.Factory.Get( "FriendService" ).New() />
						
					</cfcatch>
				</cftry>
				
				
				<!--- Update friend properties. --->
				<cfset LOCAL.StatusFriend.TwitterID = LOCAL.StatusData.Sender.id />
				<cfset LOCAL.StatusFriend.Name = LOCAL.StatusData.Sender.name />
				<cfset LOCAL.StatusFriend.ScreenName = LOCAL.StatusData.Sender.screen_name />
				<cfset LOCAL.StatusFriend.PhotoURL = LOCAL.StatusData.Sender.profile_image_url />
				<cfset LOCAL.StatusFriend.TwitterURL = "http://www.twitter.com/#LOCAL.StatusData.Sender.screen_name#" />
				
				<!--- Save the friend data. --->
				<cfset VARIABLES.Factory.Get( "FriendService" ).Save( LOCAL.StatusFriend ) />
				
					
				<!--- 
					ASSERT: At this point, LOCAL.StatusFriend has the most up to date 
					data for the current friend whether or not this was a new friend.
				--->
				
				
				<!--- Insert into direct messages. --->
				
				<!--- Create a new status for this status / group combination. --->
				<cfset LOCAL.Status = VARIABLES.Factory.Get( "StatusService" ).New() />
				
				<!--- Set status values. --->
				<cfset LOCAL.Status.TwitterID = LOCAL.StatusData.id />
				<cfset LOCAL.Status.Status = THIS.ParseStatusText( LOCAL.StatusData.text ) />
				<cfset LOCAL.Status.IsGeneral = false />
				<cfset LOCAL.Status.IsReply = false />
				<cfset LOCAL.Status.IsDirect = true />
				<cfset LOCAL.Status.IsSnoozed = false />
				<cfset LOCAL.Status.FriendID = LOCAL.StatusFriend.ID />
				<cfset LOCAL.Status.DateCreated = LOCAL.StatusData.created_at />
				
				<!--- Save the status. --->
				<cfset VARIABLES.Factory.Get( "StatusService" ).Save( LOCAL.Status ) />
						
			</cfloop>
					
						
			<!--- Catch misc errors. --->
			<cfcatch>
				<!--- We just dont' want errors to bubble up. --->
				
				<!--- Log error. --->
				<cfset VARIABLES.Factory.Get( "LoggingService" ).LogData(
					"UpdateDirectStatuses() Error",
					CFCATCH,
					"error"
					) />
					
			</cfcatch>			
		</cftry>
			
			
		<!--- Return out. --->
		<cfreturn />
	</cffunction>

		
	<cffunction 
		name="UpdateGeneralStatuses" 
		access="public" 
		returntype="void" 
		output="false" 
		hint="I get the latest GENERAL Twitter status list and update the local database.">
		
		<!--- Define the local scope. --->
		<cfset var LOCAL = {} />
		
		<!--- Get the last ID. --->
		<cfset LOCAL.LastTwitterID = VARIABLES.Factory.Get( "StatusService" ).GetMostRecentTwitterID(
			IsDirect = false
			) />
		
		<!--- Build the parameters to pass along. --->
		<cfset LOCAL.Parameters = [] />
		
		<!--- Check for twitter ID. --->
		<cfif LOCAL.LastTwitterID>
		
			<!--- Define parameter. --->
			<cfset LOCAL.SinceIDParameter = {
				Name = "since_id",
				Value = LOCAL.LastTwitterID
				} />
				
			<!--- Add to parameter collection for API request. --->
			<cfset ArrayAppend(
				LOCAL.Parameters,
				LOCAL.SinceIDParameter
				) />
		
		</cfif>
		
		
		<!--- Create count parameter. --->
		<cfset LOCAL.CountParameter = {
			Name = "count",
			Value = 200
			} />
			
		<!--- Add to parameter collection for API request. --->
		<cfset ArrayAppend(
			LOCAL.Parameters,
			LOCAL.CountParameter
			) />
		
				
		<!--- Make sure this doesn't break in any way. --->
		<cftry>
		
			<!--- Send the request. --->
			<cfset LOCAL.TwitterResponse = THIS.SubmitTwitterAPIRequest(
				RequestMethod = "get",
				Resource = "statuses/friends_timeline",
				ID = "",
				Parameters = LOCAL.Parameters
				) />
				
				
			<!--- 
				Check to make sure we have appropriate data. Sometimes, this comes 
				back as a struct (should be an array).
			--->
			<cfif NOT IsArray( LOCAL.TwitterResponse.Data )>
			
				<!--- Log error. --->
				<cfset VARIABLES.Factory.Get( "LoggingService" ).LogData(
					"UpdateGeneralStatuses() Bad Twitter Data",
					LOCAL.TwitterResponse,
					"error"
					) />
			
				<!--- This was supposed to be data. --->
				<cfreturn />
			
			</cfif>
				
				
				
			<!--- Get the groups. --->
			<cfset LOCAL.Groups = VARIABLES.Factory.Get( "GroupService" ).GetAll() />
										
			
			<!--- Loop over the status data. --->
			<cfloop
				index="LOCAL.StatusData"
				array="#LOCAL.TwitterResponse.Data#">
				
				<!--- Check to make sure that this data item is actually a structure. --->
				<cfif IsStruct( LOCAL.StatusData )>
				
					
					<!--- Format date properly. Sun Oct 05 21:50:35 +0000 2008. --->
					<cfset LOCAL.DateStatusCreated = REReplaceNoCase(
						LOCAL.StatusData.created_at,
						"(\w+) (\w+ \d+) (\d+:\d+:\d+) \+\d+ (\d+)",
						"\2 \4 \3",
						"one"								
						) />
						
					<!--- Store status date back in. --->
					<cfset LOCAL.StatusData.created_at = ParseDateTime( LOCAL.DateStatusCreated ) />
					
					
					<!--- Try to get this friend based on the twitter ID. --->
					<cftry>
						<cfset LOCAL.StatusFriend = VARIABLES.Factory.Get( "FriendService" ).GetByTwitterID( LOCAL.StatusData.User.ID ) />
					
						<!--- Catch error if friend does not exist. --->
						<cfcatch>
							
							<!--- Create new friend. --->
							<cfset LOCAL.StatusFriend = VARIABLES.Factory.Get( "FriendService" ).New() />
							
						</cfcatch>
					</cftry>
					
					
					<!--- Update friend properties. --->
					<cfset LOCAL.StatusFriend.TwitterID = LOCAL.StatusData.User.id />
					<cfset LOCAL.StatusFriend.Name = LOCAL.StatusData.User.name />
					<cfset LOCAL.StatusFriend.ScreenName = LOCAL.StatusData.User.screen_name />
					<cfset LOCAL.StatusFriend.PhotoURL = LOCAL.StatusData.User.profile_image_url />
					<cfset LOCAL.StatusFriend.TwitterURL = "http://www.twitter.com/#LOCAL.StatusData.User.screen_name#" />
					
					<!--- Save the friend data. --->
					<cfset VARIABLES.Factory.Get( "FriendService" ).Save( LOCAL.StatusFriend ) />
					
					
					<!--- 
						ASSERT: At this point, LOCAL.StatusFriend has the most up to date 
						data for the current friend whether or not this was a new friend.
					--->
					
										
					<!--- Now, we have to move this status into the various different groups. --->
					
					<!--- 
						Create a default flag that this status should be included in the general 
						tweets group. If we match a group that excludes it, this flag will be updated.
					--->
					<cfset LOCAL.IncludeInGeneralTweets = true />
					
					
					<!--- Loop over the custom groups to see if this fits into any of them. --->
					<cfloop 
						index="LOCAL.Group"
						array="#LOCAL.Groups#">
						
						<!--- Set a flag to see if this status should be included. --->
						<cfset LOCAL.IncludeInGroup = false />
					
						<!--- 
							Loop over the patterns to see if they need to be included. Do this 
							in a Try/Catch in case the patterns are not valid.
						--->
						<cfloop
							index="LOCAL.Pattern"
							list="#LOCAL.Group.IncludePatterns#"
							delimiters="#Chr( 13 )##Chr( 10 )#">
							
							<cftry>
								<cfif (
									Len( Trim( LOCAL.Pattern ) ) AND
									REFindNoCase( LOCAL.Pattern, LOCAL.StatusData.text )
									)>
									
									<!--- Pattern matched. Flag for inclusion. --->
									<cfset LOCAL.IncludeInGroup = true />
									
								</cfif>
								
								<cfcatch>
									<!--- Pattern was not valid. --->
								</cfcatch>								
							</cftry>								
							
						</cfloop>
						
						
						<!--- 
							Now, we have to run the excludes to see if we need to cancel our inclusion.
							Of course, we only need to do this if we are including the status.
						--->
						<cfif LOCAL.IncludeInGroup>
						
							<!--- Loop over exclude patterns. --->
							<cfloop
								index="LOCAL.Pattern"
								list="#LOCAL.Group.ExcludePatterns#"
								delimiters="#Chr( 13 )##Chr( 10 )#">
								
								<cftry>
									<cfif (
										Len( Trim( LOCAL.Pattern ) ) AND
										REFindNoCase( LOCAL.Pattern, LOCAL.StatusData.text )
										)>
										
										<!--- Pattern matched. Flag for exclusion. --->
										<cfset LOCAL.IncludeInGroup = false />
										
									</cfif>
									
									<cfcatch>
										<!--- Pattern was not valid. --->
									</cfcatch>								
								</cftry>								
								
							</cfloop>
							
						</cfif>							
						
						
						<!--- 
							Now that we have run out patterns matching, let's check to see 
							if we need to include this. 
						--->
						<cfif LOCAL.IncludeInGroup>
						
							<!--- Create a new status for this status / group combination. --->
							<cfset LOCAL.Status = VARIABLES.Factory.Get( "StatusService" ).New() />
							
							<!--- Set status values. --->
							<cfset LOCAL.Status.TwitterID = LOCAL.StatusData.id />
							<cfset LOCAL.Status.Status = THIS.ParseStatusText( LOCAL.StatusData.text ) />
							<cfset LOCAL.Status.IsGeneral = false />
							<cfset LOCAL.Status.IsReply = false />
							<cfset LOCAL.Status.IsDirect = false />
							<cfset LOCAL.Status.IsSnoozed = false />
							<cfset LOCAL.Status.FriendID = LOCAL.StatusFriend.ID />
							<cfset LOCAL.Status.GroupID = LOCAL.Group.ID />
							<cfset LOCAL.Status.DateCreated = LOCAL.StatusData.created_at />
							
							<!--- Save the status. --->
							<cfset VARIABLES.Factory.Get( "StatusService" ).Save( LOCAL.Status ) />
							
							
							
							<!--- Check to see if this group excludes from the general tweets. --->
							<cfif LOCAL.Group.IsRemovedFromGeneralTweets>
							
								<!--- Flag for general exclusion. --->
								<cfset LOCAL.IncludeInGeneralTweets = false />
							
							</cfif>
						
						</cfif>
					
					</cfloop>
					
					
					<!--- 
						Done checking custom groups. Now, let's check the rest of the 
						standard groups (general, snoozed, reply). 
					--->
					
					
					<!--- 
						Check to see if this user is snoozed. Only bother doing this if the 
						status has not been excluded by the groups.
					--->
					<cfif (
						IsDate( LOCAL.StatusFriend.DateSnoozedUntil ) AND
						(LOCAL.StatusFriend.DateSnoozedUntil GT Now()) AND	
						LOCAL.IncludeInGeneralTweets					
						)>
						
						<!--- Insert into snoozed. --->
						
						<!--- Create a new status for this status / group combination. --->
						<cfset LOCAL.Status = VARIABLES.Factory.Get( "StatusService" ).New() />
						
						<!--- Set status values. --->
						<cfset LOCAL.Status.TwitterID = LOCAL.StatusData.id />
						<cfset LOCAL.Status.Status = THIS.ParseStatusText( LOCAL.StatusData.text ) />
						<cfset LOCAL.Status.IsGeneral = false />
						<cfset LOCAL.Status.IsReply = false />
						<cfset LOCAL.Status.IsDirect = false />
						<cfset LOCAL.Status.IsSnoozed = true />
						<cfset LOCAL.Status.FriendID = LOCAL.StatusFriend.ID />
						<cfset LOCAL.Status.GroupID = 0 />
						<cfset LOCAL.Status.DateCreated = LOCAL.StatusData.created_at />
						
						<!--- Save the status. --->
						<cfset VARIABLES.Factory.Get( "StatusService" ).Save( LOCAL.Status ) />
						
					<cfelseif LOCAL.IncludeInGeneralTweets>
				
						<!--- Include in general tweets. --->
						
						<!--- Create a new status for this status / group combination. --->
						<cfset LOCAL.Status = VARIABLES.Factory.Get( "StatusService" ).New() />
						
						<!--- Set status values. --->
						<cfset LOCAL.Status.TwitterID = LOCAL.StatusData.id />
						<cfset LOCAL.Status.Status = THIS.ParseStatusText( LOCAL.StatusData.text ) />
						<cfset LOCAL.Status.IsGeneral = true />
						<cfset LOCAL.Status.IsReply = false />
						<cfset LOCAL.Status.IsDirect = false />
						<cfset LOCAL.Status.IsSnoozed = false />
						<cfset LOCAL.Status.FriendID = LOCAL.StatusFriend.ID />
						<cfset LOCAL.Status.GroupID = 0 />
						<cfset LOCAL.Status.DateCreated = LOCAL.StatusData.created_at />
						
						<!--- Save the status. --->
						<cfset VARIABLES.Factory.Get( "StatusService" ).Save( LOCAL.Status ) />
				
					</cfif>
				
					
					<!--- Check to see if this status is a reply. --->
					<cfif FindNoCase( "@#APPLICATION.Twitter.Username#", LOCAL.StatusData.text )>
					
						<!--- Insert into reply. --->
						
						<!--- Create a new status for this status / group combination. --->
						<cfset LOCAL.Status = VARIABLES.Factory.Get( "StatusService" ).New() />
						
						<!--- Set status values. --->
						<cfset LOCAL.Status.TwitterID = LOCAL.StatusData.id />
						<cfset LOCAL.Status.Status = THIS.ParseStatusText( LOCAL.StatusData.text ) />
						<cfset LOCAL.Status.IsGeneral = false />
						<cfset LOCAL.Status.IsReply = true />
						<cfset LOCAL.Status.IsDirect = false />
						<cfset LOCAL.Status.IsSnoozed = false />
						<cfset LOCAL.Status.FriendID = LOCAL.StatusFriend.ID />
						<cfset LOCAL.Status.GroupID = 0 />
						<cfset LOCAL.Status.DateCreated = LOCAL.StatusData.created_at />
						
						<!--- Save the status. --->
						<cfset VARIABLES.Factory.Get( "StatusService" ).Save( LOCAL.Status ) />
						
					</cfif>
					
				</cfif>				
				
			</cfloop>
			
			
			<!--- Catch any errors. --->
			<cfcatch>
		
				<!--- Log error. --->
				<cfset VARIABLES.Factory.Get( "LoggingService" ).LogData(
					"UpdateGeneralStatuses() Error",
					CFCATCH,
					"error"
					) />
					
			</cfcatch>
		
		</cftry>
			
			
		<!--- Return out. --->
		<cfreturn />
	</cffunction>
	
	
	<cffunction 
		name="UpdateStatuses" 
		access="public" 
		returntype="void" 
		output="false" 
		hint="I get the latest Twitter status list and update the local database.">
		
		<!--- Define the local scope. --->
		<cfset var LOCAL = {} />
		
		<!--- Make sure this doesn't break in any way. --->
		<cftry>
			
			<!--- 
				Single thread this request. We want to make sure that we don't hit the 
				Twitter client too often. We are going to limit this to once per minute.
			--->
			<cflock
				name="KinkyTwits.TwitterService.UpdateStatuses"
				type="exclusive"
				timeout="30">
				
				<!--- 
					Check to see if we can make this request. Has it been at least 60 
					seconds since the last request.
				--->
				<cfif (
					IsDate( VARIABLES.Instance.DateTwitterLastPinged ) AND
					(DateDiff( "s", VARIABLES.Instance.DateTwitterLastPinged, Now() ) LT Fix( 60 * 60 / VARIABLES.Instance.APILimit ))
					)>
				
					<!--- We are pinging too often, return out. --->
					<cfreturn />
				
				</cfif>
				
				
				<!--- Now that we have pinged, store the date of the last ping. --->
				<cfset VARIABLES.Instance.DateTwitterLastPinged = Now() />
				
				<!--- Update the ping request. --->
				<cfset VARIABLES.Instance.TwitterPingCount++ />
				
				
				<!--- 
					Check to see which kind of twitter request we are going to make. Every 10'th 
					request, we are going to get a direct response.
				--->
				<cfif ((VARIABLES.Instance.TwitterPingCount MOD 10) EQ 9)>
					
					<!--- Get DIRECT statuses. --->
					<cfset THIS.UpdateDirectStatuses() />
				
				<cfelse>
				
					<!--- Get general statuses. --->
					<cfset THIS.UpdateGeneralStatuses() />
				
				</cfif>
				
			</cflock>
			
			
			<!--- Catch misc errors. --->
			<cfcatch>
				<!--- We just dont' want errors to bubble up. --->
				
				<!--- Log error. --->
				<cfset VARIABLES.Factory.Get( "LoggingService" ).LogData(
					"UpdateStatuses() Outer Error",
					CFCATCH,
					"error"
					) />
					
			</cfcatch>			
		</cftry>
			
		<!--- Return out. --->
		<cfreturn />
	</cffunction>
	
</cfcomponent>