
// This is the constructor for our API object.
function KinkyTwits(){
	var objSelf = this;
	
	// The current list and group elements.
	this.CurrentStatusList = $( [] );
	this.CurrentGroup = $( [] );
	
	// A cache of DOM element references.
	this.DOMReferences = {
		Body: $( "body:first" ),
		StatusForm: $( "#status-form" ),
		StatusFormInput: $( "#status-form input.status" ),
		StatusFormCharacterCount: $( "#status-form div.characters-remaining span.count" ),
		StatusFormUrlInput: $( "#status-form input.url" ),
		StatusFormUrlButton: $( "#status-form button.shorten" ),
		StatusListContainer: $( "#status-list-container" ),
		GroupList: $( "#groups-list" ),
		AddNewGroup: $( "#add-new-group" ),
		AddNewGroupForm: null,
		
		// Element templates.
		GroupTemplate: $( "#element-templates li.group" ),
		GroupFormTemplate: $( "#element-templates form.group-form" ),
		StatusTemplate: $( "#element-templates li.status" ),
		StatusListTemplate: $( "#element-templates ol.status-list" )
		};

	// The possible request objects (will be used to test business).
	this.AJAXRequests = {
		EditGroup: null,
		DeleteGroup: null,
		SubmitGroupForm: null,
		PopulateGroupList: null,
		SubmitAddNewGroupForm: null,
		SubmitStatusForm: null,
		UpdateStatusList: null,
		SnoozeFriend: null,
		UnsnoozeFriend: null,
		ShortenUrl: null
		};
		
	// Hook AJAX error handler.
	this.DOMReferences.Body.ajaxError( 
		function( objEvent, objRequest, objSettings ){
			// Pass event onto API handler.
			objSelf.AJAXErrorHandler( objEvent, objRequest, objSettings );
		}
		);
		
		
	// Hook up window blur and focus listener.
	$( window )
		.blur( 
			function(){
				objSelf.WindowBlurHandler()
			}
			)
		.focus( 
			function(){
				objSelf.WindowFocusHandler()
			}
			)
	;	
	
		
	// The max number of status to display.
	this.MaxStatusCountPerList = 75;
	
	// ID of the last status brought back.
	this.LastStatusID = 0;
	
	// This is the number of unread messages (since the window was blurred).
	this.UnreadMessageCount = 0;
	
	// Store a flag for window focus.
	this.WindowFocused = true;
		
	// Initialize page elements.
	this.InitializeStatusForm();
	this.InitializeAddNewGroup();
	this.InitializeGroupList();	 // Once this finishes, the list will update.
		
	// Return this pointer.
	return( this );
}


// Add the given group the DOM.
KinkyTwits.prototype.AddGroup = function( objGroup ){
	var objSelf = this;
	
	// Create a new status element from template.
	var jGroup = this.DOMReferences.GroupTemplate.clone();
	
	// Set the group ID.
	jGroup.attr( "id", (objGroup.ID + "-group") );
	
	// Set the group label and status count.
	jGroup.find( "div.header a.label span.label-text" ).text( objGroup.NAME );
	jGroup.find( "div.header a.label span.status-count" ).text( "(0)" );
	
	// Get the groups buttons.
	var jLink = jGroup.find( "div.header a.label" );
	var jEdit = jGroup.find( "p.group-tools a.edit" );
	var jDelete = jGroup.find( "p.group-tools a.delete" );
	
	// Hook up the label.
	jLink
		.attr( "href", "javascript:void( 0 )" )
		.click(
			function( objEvent ){
				// Show group.
				objSelf.ShowGroup( jGroup );
				
				// Blur the link.
				this.blur();
				
				// Prevent default.
				objEvent.preventDefault();
				return( false );
			}
			)
	;
	
	// Hook up the edit button.
	jEdit
		.attr( "href", "javascript:void( 0 )" )
		.click(
			function( objEvent ){
				// Edit group.
				objSelf.EditGroup( jGroup );
				
				// Prevent default.
				objEvent.preventDefault();
				return( false );
			}
			)
	;
	
	// Hook up the delete button.
	jDelete
		.attr( "href", "javascript:void( 0 )" )
		.click(
			function( objEvent ){
				// Make sure this was done on purpose.
				if (confirm( "Delete this group?" )){
					objSelf.DeleteGroup( jGroup )
				}
							
				// Prevent default.
				objEvent.preventDefault();
				return( false );
			}
			)
	;
		
	// Add the group to the DOM.
	this.DOMReferences.GroupList.append( jGroup );
	
	// Add the status list.
	this.AddStatusList( objGroup.ID );
}


// This adds the status list.
KinkyTwits.prototype.AddStatusList = function( strGroupID ){
	var jStatusList = this.CreateNewStatusListElement( strGroupID );
	
	// Hide the status list.
	jStatusList.hide();
	
	// Add the status list the dom.
	this.DOMReferences.StatusListContainer.append( jStatusList );
}


// This handles any AJAX errors that get thrown from the API.
KinkyTwits.prototype.AJAXErrorHandler = function( objEvent, objRequest, objSettings ){
	// Create a "failure" API response.
	var objResponse = {
		SUCCESS: false,
		ERRORS: [ "An unknown API error occurred!" ],
		DATA: ""
		};
	
	// Check to see if we can match the request that errored.
	if (objRequest == this.AJAXRequests.SubmitStatusForm){
	
		this.SubmitStatusFormHandler( objResponse );
	
	} else if (objRequest == this.AJAXRequests.SubmitAddNewGroupForm){
	
		this.SubmitAddNewGroupFormHandler( objResponse );
	
	} else if (objRequest == this.AJAXRequests.EditGroup){
	
		this.EditGroupHandler( objResponse );
		
	} else if (objRequest == this.AJAXRequests.DeleteGroup){
	
		this.DeleteGroupHandler( objResponse );
		
	} else if (objRequest == this.AJAXRequests.SubmitGroupForm){
	
		this.SubmitGroupFormHandler( objResponse );
	
	} else if (objRequest == this.AJAXRequests.SnoozeFriend){
	
		this.SnoozeFriendHandler( objResponse );
	
	} else if (objRequest == this.AJAXRequests.UnsnoozeFriend){
	
		this.UnsnoozeFriendHandler( objResponse );
		
	} else if (objRequest == this.AJAXRequests.UpdateStatusList){
	
		this.UpdateStatusListHandler( objResponse );
		
	} else if (objRequest == this.AJAXRequests.ShortenUrl){
	
		this.ShortenUrlHandler( objResponse );
		
	}
}


// This alerts the user to the given API errors.
KinkyTwits.prototype.AlertAPIErrors = function( arrErrors ){
	var strMessage = "Please review the following:\n\n";
	
	// Add the error messages.
	$.map(
		arrErrors,
		function( strError ){
			strMessage += (strError + "\n" );
		}
		);
	
	// Alert compiled message.
	this.AlertUser( strMessage );
}


// This alerts the user to the given message.
KinkyTwits.prototype.AlertUser = function( strMessage ){
	alert( strMessage );
}


// This creates and returns a new status element based on the given 
// status object.
KinkyTwits.prototype.CreateNewStatusElement = function( objStatus ){
	var objSelf = this;
	
	// Create a new status element from template.
	var jStatus = this.DOMReferences.StatusTemplate.clone();
	
	// Insert text elements.
	jStatus.find( "p.status-text" ).html( objStatus.STATUS );
	jStatus.find( "p.status-author" ).html( objStatus.AUTHORED );
	
	// Update the photo src.
	jStatus.find( "div.member-photo img" ).attr( "src", objStatus.FRIEND_PHOTO_URL );

	// Get buttons.
	var jReply = jStatus.find( "p.status-tools a.reply" );
	var jDirect = jStatus.find( "p.status-tools a.direct" );
	var jSnooze = jStatus.find( "p.status-tools a.snooze" );
	
	// Hook up reply.
	jReply
		.attr( "href", "javascript:void( 0 )" )
		.click(
			function( objEvent ){
				// Check to see if this status is a Direct message. If so, 
				// let's recommend that the reply id Direct as well.
				if (
					objStatus.IS_DIRECT &&
					confirm( "This was a DIRECT message. Would you rather reply with a DIRECT message?" )
					){
				
					// Send a direct message.
					objSelf.InitializeStatusMessage( "Direct", objStatus.FRIEND_SCREEN_NAME );
				
				} else {
					
					// They opted for a standard reply.
					objSelf.InitializeStatusMessage( "Reply", objStatus.FRIEND_SCREEN_NAME );
				
				}
				
				// Kill default click.
				objEvent.preventDefault();
				return( false );
			}
			)
	;
	
	// Hook up Direct.
	jDirect
		.attr( "href", "javascript:void( 0 )" )
		.click(
			function( objEvent ){
				// Start the direct message.
				objSelf.InitializeStatusMessage( "Direct", objStatus.FRIEND_SCREEN_NAME );
				
				// Kill default click.
				objEvent.preventDefault();
				return( false );
			}
			)
	;
	
	// Check to see if this friend is current snoozed or not.
	if (objStatus.FRIEND_IS_SNOOZED){
	
		// Change the link text to reflect UNsnoozing.
		jSnooze.html( "Unsnooze (" + objStatus.SNOOZE_TIME + ")" );
	
		// Hook up Unsnooze.
		jSnooze
			.attr( "href", "javascript:void( 0 )" )
			.click(
				function( objEvent ){
					if (confirm( "Unsnooze " + objStatus.FRIEND_NAME + "?" )){
						objSelf.UnsnoozeFriend( objStatus.FRIEND_ID );
					}
					
					// Kill default click.
					objEvent.preventDefault();
					return( false );
				}
				)
		;	
	
	} else {
	
		// Hook up Snooze.
		jSnooze
			.attr( "href", "javascript:void( 0 )" )
			.click(
				function( objEvent ){
					if (confirm( "Snooze " + objStatus.FRIEND_NAME + " for an hour?" )){
						objSelf.SnoozeFriend( objStatus.FRIEND_ID );
					}
					
					// Kill default click.
					objEvent.preventDefault();
					return( false );
				}
				)
		;	
	
	}
	
	
	// Return status.
	return( jStatus );	
}


// This creates and returns a new status list element with the given group ID.
KinkyTwits.prototype.CreateNewStatusListElement = function( strGroupID ){
	var objSelf = this;
	
	// Create a new status list element from template.
	var jStatusList = this.DOMReferences.StatusListTemplate.clone();
	
	// Set the list ID (use hack to make sure its a string).
	jStatusList.attr( "id", ((strGroupID + "").toLowerCase() + "-status-list") );
	
	// Return status.
	return( jStatusList );	
}


// This attempts to delete the given group.
KinkyTwits.prototype.DeleteGroup = function( jGroup ){
	var objSelf = this;

	// Check to see if we have a current API request.
	if (!this.AJAXRequests.DeleteGroup){
	
		// Submit request to API.
		this.AJAXRequests.DeleteGroup = $.getJSON(
			"./api/Groups.cfc",
			{
				Method: "DeleteGroup",
				ID: this.GetGroupIDFromGroup( jGroup )
			},
			function( objResponse ){
				objSelf.DeleteGroupHandler( objResponse, jGroup );
			}
			);
	
	} else {
		
		// There is an existing request. Alert exception.
		this.AlertUser( "There is a group being deleted. Please be patient." );
	
	}
}


// This handles the delete group response.
KinkyTwits.prototype.DeleteGroupHandler = function( objResponse, jGroup ){
	// Check to see if the request was successful.
	if (objResponse.SUCCESS){
	
		// The API request was successful. Remove the group
		// and the status list associated with it.
		jGroup.remove();
		
		// Remove status list.
		this.DeleteStatusList( this.GetGroupIDFromGroup( jGroup ) );
				
	} else {
	
		// The response failed. Alert errors.
		this.AlertAPIErrors( objResponse.ERRORS );
	
	}

	// Clear the current request.
	this.AJAXRequests.DeleteGroup = null;
}


// This deletes the status list with the given group ID from the dom.
KinkyTwits.prototype.DeleteStatusList = function( strGroupID ){
	this.DOMReferences.StatusListContainer.find( "#" + strGroupID + "-status-list" ).remove();
}


// This attempts to gather the given group data for editing 
// (assuming that there is not an existing request for the form).
KinkyTwits.prototype.EditGroup = function( jGroup ){
	var objSelf = this;
		
	// Check to see if there is already a visible form.
	if (jGroup.find( "form" ).length){
		return;
	}
		
	// Check to see if we have a current API request.
	if (!this.AJAXRequests.EditGroup){
	
		// Submit request to API.
		this.AJAXRequests.EditGroup = $.getJSON(
			"./api/Groups.cfc",
			{
				Method: "GetGroup",
				ID: this.GetGroupIDFromGroup( jGroup )
			},
			function( objResponse ){
				objSelf.EditGroupHandler( objResponse, jGroup );
			}
			);
	
	} else {
		
		// There is an existing form submission that we are waiting
		// on. Alert exception.
		this.AlertUser( "The group data is being gathered. Please be patient." );
	
	}
}


// This handles the existing group response.
KinkyTwits.prototype.EditGroupHandler = function( objResponse, jGroup ){
	var objSelf = this; 
	
	// Check to see if the request was successful.
	if (objResponse.SUCCESS){
	
		// Create a form object.
		var jForm = this.DOMReferences.GroupFormTemplate.clone();
		
		// Set form input values.
		jForm.find( "input[ name = 'id' ]" ).val( objResponse.DATA.ID );
		jForm.find( "input[ name = 'name' ]" ).val( objResponse.DATA.NAME );
		jForm.find( "input[ name = 'is_removed_from_general_tweets' ]" ).get( 0 ).checked = (objResponse.DATA.ISREMOVEDFROMGENERALTWEETS != 0);
		jForm.find( "textarea[ name = 'include_patterns' ]" ).val( objResponse.DATA.INCLUDEPATTERNS );
		jForm.find( "textarea[ name = 'exclude_patterns' ]" ).val( objResponse.DATA.EXCLUDEPATTERNS );
	
		// Hook up form handler.
		jForm.submit(
			function( objEvent ){
				objSelf.SubmitGroupForm( jGroup );
			
				// Prevent default.
				objEvent.preventDefault();
				return( false );
			}
			);
			
		// Hook up cancel button.
		jForm
			.find( "p.buttons button.cancel" )
			.click(
				function( objEvent ){
					// Remove the form completely.
					jForm.remove();
									
					// Prevent default.
					objEvent.preventDefault();
					return( false );
				}
				)
		;
	
		// Add the form to the group and show it.
		jGroup.append( jForm );
		jForm.show();
	
	} else {
	
		// The response failed. Alert errors.
		this.AlertAPIErrors( objResponse.ERRORS );
	
	}

	// Clear the current request.
	this.AJAXRequests.EditGroup = null;
}


// This gets the group ID from the given group.
KinkyTwits.prototype.GetGroupIDFromGroup = function( jGroup ){
	return(
		jGroup.attr( "id" ).replace( new RegExp( "-group$" ), "" ) 
		);
}


// This calculates the reamining character input limit.
KinkyTwits.prototype.GetRemainingCharacterCount = function( jInput ){
	var strValue = jInput.val();
	var intAllowedCount = 140;
	
	// Check to see if this is a direct message or a standard message.
	// A direct message has special length and parsing usage.
	if ((new RegExp( "^D ", "i" )).test( strValue )){
		
		// A direct message can only allow 140 characters.
		// intAllowedCount = 140;
		
		// Strip out the leading D and username.
		strValue = strValue.replace( new RegExp( "^D [^ ]+ *", "i" ), "" );
	
	}
	
	// At this point, we have removed any special values from the string.
	// Now, let's replace HTML characters.
	strValue = strValue.replace(
		new RegExp( "<|>|&", "gi" ),
		function( $0 ){
			switch( $0 ){
				case "<":
					return( "&lt;" );
					break;
				case ">":
					return( "&gt;" );
					break;
				case "&":
					return( "&amp;" );
					break;
			}
		}
		);
		
	// Return the length of the given value taken from the allowable
	// number of characters for this type of message.
	return( intAllowedCount - strValue.length );
}


// This clears the status message field and initializes it to 
// send the given type of message.
KinkyTwits.prototype.InitializeStatusMessage = function( strType, strUsername ){
	// Check to see what type of message it is.
	switch (strType){
		// For a reply message, we don't necessarily want to erase what's
		// there because there might be multiple people that we are 
		// replying to.
		case "Reply":
			this.DOMReferences.StatusFormInput
				.val( this.DOMReferences.StatusFormInput.val() + "@" + strUsername + " " )
				.get( 0 ).focus()
			;
			break;
			
		// For a direct message, let's erase the existing field and then
		// add the direct message starter.
		case "Direct":
			this.DOMReferences.StatusFormInput
				.val( "D " + strUsername + " " )
				.get( 0 ).focus()
			;
			break;
	}
}


// This initializes the Add New Group form functionality.
KinkyTwits.prototype.InitializeAddNewGroup = function(){
	var objSelf = this;
	var jLink = this.DOMReferences.AddNewGroup.find( "div.header a.label" );
	
	// Create the form element.
	this.DOMReferences.AddNewGroupForm = this.DOMReferences.GroupFormTemplate.clone();
	
	// Add new form to DOM.
	this.DOMReferences.AddNewGroup.append( this.DOMReferences.AddNewGroupForm );
	
	// Hook up the toggle link.
	jLink
		.attr( "href", "javascript:void( 0 )" )
		.click(
			function( objEvent ){
				// Check to see if we are hiding or showing the form.
				if (objSelf.DOMReferences.AddNewGroupForm.is( ":visible" )){
					objSelf.ToggleAddNewGroupForm( "Hide" );
				} else {
					objSelf.ToggleAddNewGroupForm( "Show" );
				}
				
				// Blur the link.
				jLink.blur();
				
				// Prevent default.
				objEvent.preventDefault();
				return( false );
			}
			)
	;
	
	// Hook up form handler.
	this.DOMReferences.AddNewGroupForm.submit(
		function( objEvent ){
			objSelf.SubmitAddNewGroupForm();
		
			// Prevent default.
			objEvent.preventDefault();
			return( false );
		}
		);
		
	// Hook up cancel button.
	this.DOMReferences.AddNewGroupForm
		.find( "p.buttons button.cancel" )
		.click(
			function( objEvent ){
				// Hide form.
				objSelf.ToggleAddNewGroupForm( "Hide" );
				
				// Prevent default.
				objEvent.preventDefault();
				return( false );
			}
			)
	;
}


// This initializes the group list.
KinkyTwits.prototype.InitializeGroupList = function(){
	var objSelf = this;
	
	// Hook up the linkes on the built-in groups.
	this.DOMReferences.GroupList
		.find( "li.group" )
		.each(
			function( intI ){
				var jGroup = $( this );
				var jLink = jGroup.find( "div.header a.label" );
				
				// Hook up link.
				jLink
					.attr( "href", "javascript:void( 0 )" )
					.click(
						function( objEvent ){
							// Edit group.
							objSelf.ShowGroup( jGroup );
							
							// Blur the link.
							this.blur();
							
							// Prevent default.
							objEvent.preventDefault();
							return( false );
						}
						)
				;
				
				// Add the status list.
				objSelf.AddStatusList( objSelf.GetGroupIDFromGroup( jGroup ) );
			}
			)
	;

	// Populate the rest of the group lists.
	this.PopulateGroupList();
	
	// Show the first group.
	this.ShowGroup( this.DOMReferences.GroupList.find( "li.group:first" ) );
}


// This initializes the tweet form.
KinkyTwits.prototype.InitializeStatusForm = function(){
	var objSelf = this;
	
	// Hook up the tweet form to submit back to API.
	this.DOMReferences.StatusForm.submit(
		function( objEvent ){
			objSelf.SubmitStatusForm();
			
			// Prevent the default.
			objEvent.preventDefault();
			return( false );
		}
		);
		
	// Hook up key handler for input field.
	this.DOMReferences.StatusFormInput.keyup(
		function( objEvent ){
			// Set the character count.
			objSelf.DOMReferences.StatusFormCharacterCount.text(
				objSelf.GetRemainingCharacterCount( objSelf.DOMReferences.StatusFormInput )
				);
		}
		);
		
	// When the input is focused, calculate the chatacter count.
	this.DOMReferences.StatusFormInput.focus(
		function( objEvent ){
			// Set the character count.
			objSelf.DOMReferences.StatusFormCharacterCount.text(
				objSelf.GetRemainingCharacterCount( objSelf.DOMReferences.StatusFormInput )
				);
		}
		);
		
	// Hook up key handler for input field so that if the key is a 
	// RETURN, we won't submit the form.
	this.DOMReferences.StatusFormUrlInput.keypress(
		function( objEvent ){
			if ((objEvent.keyCode || objEvent.charCode) == 13){
				// Shorten URL rather than submitting status form.
				objSelf.ShortenUrl();
				
				// Prevent default.
				objEvent.preventDefault();
				return( false );
				
			}			
		}
		);
		
	// Hook up the shorten URL button.
	this.DOMReferences.StatusFormUrlButton.click(
		function( objEvent ){
			objSelf.ShortenUrl();
		
			// Prevent default.
			objEvent.preventDefault();
			return( false );
		}
		);
}


// This attempts to get the list of groups (assuming that there is
// not an existing request).
KinkyTwits.prototype.PopulateGroupList = function(){
	var objSelf = this;

	// Check to see if we have a current API request.
	if (!this.AJAXRequests.PopulateGroupList){
	
		// Submit status message to API.
		this.AJAXRequests.PopulateGroupList = $.getJSON(
			"./api/Groups.cfc",
			{
				Method: "GetGroups"
			},
			function( objResponse ){
				objSelf.PopulateGroupListHandler( objResponse );
			}
			);
	
	} else {
		
		// There is an existing request that we are waiting
		// on. Alert exception.
		this.AlertUser( "The group list is being gathered. Please be patient." );
	
	}
}


// This handles the group list response.
KinkyTwits.prototype.PopulateGroupListHandler = function( objResponse ){
	var objSelf = this;
	
	// Check to see if the request was successful.
	if (objResponse.SUCCESS){
		
		// Loop over the groups.
		$.map(
			objResponse.DATA,
			function( objGroup, intIndex ){
				objSelf.AddGroup( objGroup );
			}
			);
	
		// At this point, the group list has been iniatialized. We can now 
		// populate the status list.
		this.UpdateStatusList();
	
	} else {
	
		// The response failed. Alert errors.
		this.AlertAPIErrors( objResponse.ERRORS );
	
	}

	// Clear the current request.
	this.AJAXRequests.PopulateGroupList = null;
}


// This attempts to shorten the given (assuming that there is
// not an existing request).
KinkyTwits.prototype.ShortenUrl = function(){
	var objSelf = this;

	// Check to see if we have a current API request.
	if (!this.AJAXRequests.ShortenUrl){
	
		// Submit status message to API.
		this.AJAXRequests.ShortenUrl = $.getJSON(
			"./api/URL.cfc",
			{
				Method: "ShortenUrl",
				Url: this.DOMReferences.StatusFormUrlInput.val()
			},
			function( objResponse ){
				objSelf.ShortenUrlHandler( objResponse );
			}
			);
	
	} else {
		
		// There is an existing request that we are waiting
		// on. Alert exception.
		this.AlertUser( "The URL is being shortened. Please be patient." );
	
	}
}


// This handles the shorten URL response.
KinkyTwits.prototype.ShortenUrlHandler = function( objResponse ){
	var objSelf = this;
	
	// Check to see if the request was successful.
	if (objResponse.SUCCESS){
		
		// Add the URL to the status box.
		this.DOMReferences.StatusFormInput
			.val(
				this.DOMReferences.StatusFormInput.val() + 
				" " +
				objResponse.DATA
				)
			.focus()		
		;
		
		// Clear the url box.
		this.DOMReferences.StatusFormUrlInput.val( "" );
	
	} else {
	
		// The response failed. Alert errors.
		this.AlertAPIErrors( objResponse.ERRORS );
	
	}

	// Clear the current request.
	this.AJAXRequests.ShortenUrl = null;
}


// This shows the given group.
KinkyTwits.prototype.ShowGroup = function( jGroup ){
	var jStatusList = this.DOMReferences.StatusListContainer.find( "#" + this.GetGroupIDFromGroup( jGroup ) + "-status-list" );
	
	// Hide the current selections.
	this.CurrentStatusList.hide();
	this.CurrentGroup.removeClass( "selected" );
	
	// Show the current selection.
	jGroup.addClass( "selected" );
	jGroup.removeClass( "new-statuses" );		
	jStatusList.show();
	
	// Store currenct selection.
	this.CurrentGroup = jGroup;
	this.CurrentStatusList = jStatusList;
}


// This attempts to snooze a friend (assuming that there is
// not an existing request for the form).
KinkyTwits.prototype.SnoozeFriend = function( strID ){
	var objSelf = this;

	// Check to see if we have a current API request.
	if (!this.AJAXRequests.SnoozeFriend){
	
		// Submit status message to API.
		this.AJAXRequests.SnoozeFriend = $.getJSON(
			"./api/Friends.cfc",
			{
				Method: "SnoozeFriend",
				id: strID
			},
			function( objResponse ){
				objSelf.SnoozeFriendHandler( objResponse );
			}
			);
	
	} else {
		
		// There is an existing form submission that we are waiting
		// on. Alert exception.
		this.AlertUser( "There is a snooze being transmitted. Please be patient." );
	
	}
}


// This handles the snooze friend response.
KinkyTwits.prototype.SnoozeFriendHandler = function( objResponse ){
	// Check to see if the request was successful.
	if (objResponse.SUCCESS){
	
		// Is there anything to do?
		alert( "Friend has been snoozed." )
	
	} else {
	
		// The response failed. Alert errors.
		this.AlertAPIErrors( objResponse.ERRORS );
	
	}

	// Clear the current request.
	this.AJAXRequests.SnoozeFriend = null;
}


// This attempts to submit the add new group form (assuming that there
// is not an existing request for the form).
KinkyTwits.prototype.SubmitAddNewGroupForm = function(){
	var objSelf = this;

	// Check to see if we have a current API request.
	if (!this.AJAXRequests.SubmitAddNewGroupForm){
	
		// Submit request to API.
		this.AJAXRequests.SubmitAddNewGroupForm = $.getJSON(
			"./api/Groups.cfc",
			{
				Method: "SaveGroup",
				ID: this.DOMReferences.AddNewGroupForm.find( "input[ name = 'id' ]" ).val(),
				Name: this.DOMReferences.AddNewGroupForm.find( "input[ name = 'name' ]" ).val(),
				IsRemovedFromGeneralTweets: this.DOMReferences.AddNewGroupForm.find( "input[ name = 'is_removed_from_general_tweets' ]" )[ 0 ].checked,
				IncludePatterns: this.DOMReferences.AddNewGroupForm.find( "textarea[ name = 'include_patterns' ]" ).val(),
				ExcludePatterns: this.DOMReferences.AddNewGroupForm.find( "textarea[ name = 'exclude_patterns' ]" ).val()
			},
			function( objResponse ){
				objSelf.SubmitAddNewGroupFormHandler( objResponse );
			}
			);
	
	} else {
		
		// There is an existing form submission that we are waiting
		// on. Alert exception.
		this.AlertUser( "There is a new group being added. Please be patient." );
	
	}
}


// This handles the add new form submission response.
KinkyTwits.prototype.SubmitAddNewGroupFormHandler = function( objResponse ){
	// Check to see if the request was successful.
	if (objResponse.SUCCESS){
	
		// The API request was successful. Now, we have to add the
		// new group element and reset the form.
		
		// Add new group element.
		this.AddGroup( objResponse.DATA );
		
		// Hide the group form.
		this.ToggleAddNewGroupForm( "Hide" );
	
	} else {
	
		// The response failed. Alert errors.
		this.AlertAPIErrors( objResponse.ERRORS );
	
	}

	// Clear the current request.
	this.AJAXRequests.SubmitAddNewGroupForm = null;
}


// This attempts to submit the existing group form (assuming that there
// is not an existing request for the form).
KinkyTwits.prototype.SubmitGroupForm = function( jGroup ){
	var objSelf = this;
	var jForm = jGroup.find( "form" );
	
	// Check to see if we have a current API request.
	if (!this.AJAXRequests.SubmitGroupForm){
	
		// Submit request to API.
		this.AJAXRequests.SubmitGroupForm = $.getJSON(
			"./api/Groups.cfc",
			{
				Method: "SaveGroup",
				ID: jForm.find( "input[ name = 'id' ]" ).val(),
				Name: jForm.find( "input[ name = 'name' ]" ).val(),
				IsRemovedFromGeneralTweets: jForm.find( "input[ name = 'is_removed_from_general_tweets' ]" )[ 0 ].checked,
				IncludePatterns: jForm.find( "textarea[ name = 'include_patterns' ]" ).val(),
				ExcludePatterns: jForm.find( "textarea[ name = 'exclude_patterns' ]" ).val()
			},
			function( objResponse ){
				objSelf.SubmitGroupFormHandler( objResponse, jGroup );
			}
			);
	
	} else {
		
		// There is an existing form submission that we are waiting
		// on. Alert exception.
		this.AlertUser( "The group data is being saved. Please be patient." );
	
	}
}


// This handles the existing form submission response.
KinkyTwits.prototype.SubmitGroupFormHandler = function( objResponse, jGroup ){
	// Check to see if the request was successful.
	if (objResponse.SUCCESS){
	
		// The API request was successful. Now, we have to update
		// the group display.
		jGroup.find( "div.header a.label span.label-text" ).html( objResponse.DATA.NAME );
		
		// Remove the form.
		jGroup.find( "form" ).remove();		
		
	} else {
	
		// The response failed. Alert errors.
		this.AlertAPIErrors( objResponse.ERRORS );
	
	}

	// Clear the current request.
	this.AJAXRequests.SubmitGroupForm = null;
}


// This attempts to submit the status form (assuming that there is
// not an existing request for the form).
KinkyTwits.prototype.SubmitStatusForm = function(){
	var objSelf = this;

	// Check to see if we have a current API request.
	if (!this.AJAXRequests.SubmitStatusForm){
	
		// Submit status message to API.
		this.AJAXRequests.SubmitStatusForm = $.getJSON(
			"./api/Status.cfc",
			{
				Method: "SubmitStatus",
				Status: this.DOMReferences.StatusFormInput.val()
			},
			function( objResponse ){
				objSelf.SubmitStatusFormHandler( objResponse );
			}
			);
	
	} else {
		
		// There is an existing form submission that we are waiting
		// on. Alert exception.
		this.AlertUser( "There is a status being transmitted. Please be patient." );
	
	}
}


// This handles the status submission response.
KinkyTwits.prototype.SubmitStatusFormHandler = function( objResponse ){
	// Check to see if the request was successful.
	if (objResponse.SUCCESS){
	
		// The message went through properly. Reset the status form.
		this.DOMReferences.StatusForm.get( 0 ).reset();
		this.DOMReferences.StatusForm.focus();
		
		// Recalculate the allowable characters.
		this.DOMReferences.StatusFormCharacterCount.text(
			this.GetRemainingCharacterCount( this.DOMReferences.StatusFormInput )
			);
	
	} else {
	
		// The response failed. Alert errors.
		this.AlertAPIErrors( objResponse.ERRORS );
	
	}

	// Clear the current request.
	this.AJAXRequests.SubmitStatusForm = null;
}


// This toggles the add new group form.
KinkyTwits.prototype.ToggleAddNewGroupForm = function( strAction ){
	// Check to see which action we are performing.
	if (strAction == "Show"){
	
		// Show the form.
		this.DOMReferences.AddNewGroupForm.show();	
		
		// Focus the first element.
		this.DOMReferences.AddNewGroupForm.find( "input[ name = 'name' ]" ).focus();
	
	} else {
		
		// Reset the form.
		this.DOMReferences.AddNewGroupForm.get( 0 ).reset();
	
		// Hide the form.		
		this.DOMReferences.AddNewGroupForm.hide();
	
	}
}


// This attempts to unsnooze a friend (assuming that there is
// not an existing request for the form).
KinkyTwits.prototype.UnsnoozeFriend = function( strID ){
	var objSelf = this;

	// Check to see if we have a current API request.
	if (!this.AJAXRequests.UnsnoozeFriend){
	
		// Submit status message to API.
		this.AJAXRequests.UnsnoozeFriend = $.getJSON(
			"./api/Friends.cfc",
			{
				Method: "UnsnoozeFriend",
				id: strID
			},
			function( objResponse ){
				objSelf.UnsnoozeFriendHandler( objResponse );
			}
			);
	
	} else {
		
		// There is an existing form submission that we are waiting
		// on. Alert exception.
		this.AlertUser( "There is an unsnooze being transmitted. Please be patient." );
	
	}
}


// This handles the unsnooze friend response.
KinkyTwits.prototype.UnsnoozeFriendHandler = function( objResponse ){
	// Check to see if the request was successful.
	if (objResponse.SUCCESS){
	
		// Is there anything to do?
		alert( "Friend has been unsnoozed." )
	
	} else {
	
		// The response failed. Alert errors.
		this.AlertAPIErrors( objResponse.ERRORS );
	
	}

	// Clear the current request.
	this.AJAXRequests.UnsnoozeFriend = null;
}


// This attempts to udpate the status lists (assuming that there is
// not an existing request for the form).
KinkyTwits.prototype.UpdateStatusList = function(){
	var objSelf = this;

	// Check to see if we have a current API request.
	if (!this.AJAXRequests.UpdateStatusList){
	
		// Submit status message to API.
		this.AJAXRequests.UpdateStatusList = $.getJSON(
			"./api/Status.cfc",
			{
				Method: "GetStatuses",
				LastStatusID: this.LastStatusID
			},
			function( objResponse ){
				objSelf.UpdateStatusListHandler( objResponse );
			}
			);
	
	} else {
		
		// There is an existing form submission that we are waiting
		// on. Alert exception.
		this.AlertUser( "There is a status lists is being gathered. Please be patient." );
	
	}
}


// This handles the status list response.
KinkyTwits.prototype.UpdateStatusListHandler = function( objResponse ){
	var objSelf = this;
	
	// Check to see if the request was successful.
	if (objResponse.SUCCESS){
	
		// Loop over all the groups that came back.
		for (var strGroupID in objResponse.DATA){
			
			// Get the group and status list for this id.
			var jGroup = $( "#" + (strGroupID + "").toLowerCase() + "-group" );
			var jStatusList = $( "#" + (strGroupID + "").toLowerCase() + "-status-list" );
		
			// Loop over the statuses.
			$.map(
				objResponse.DATA[ strGroupID ],
				function( objStatus, intIndex ){
					// Create a new status element.
					var jStatus = objSelf.CreateNewStatusElement( objStatus );
					
					// Keep internal track of the max status ID requested from the server.
					objSelf.LastStatusID = Math.max( objSelf.LastStatusID, objStatus.ID );
					
					// Add this status element to the list (at the top).
					jStatusList.prepend( jStatus );
					
					// Check to see if list is visible. If not, then add the "new" 
					// class to the group button.
					if (jStatusList.is( ":hidden" )){
						jGroup.addClass( "new-statuses" );
					}
					
					// Add one to the unread message count.
					objSelf.UnreadMessageCount++;
				}
				);
				
				
			// Limit the list to max count.
			jStatusList.find( "li:gt( " + (this.MaxStatusCountPerList - 1) + " )" ).remove();

			// Update the group counts.
			jGroup.find( "div.header a.label span.status-count" ).html(
				"(" + jStatusList.find( "li" ).length + ")"
				);
		}
		
		// Update the window title.
		this.UpdateWindowTitle();
	
	} else {
	
		// The response failed. Alert errors.
		this.AlertAPIErrors( objResponse.ERRORS );
	
	}

	// Clear the current request.
	this.AJAXRequests.UpdateStatusList = null;
	
	
	// Now that this has executed succesfully, call it again (on interval).
	setTimeout(
		function(){
			objSelf.UpdateStatusList();
		},
		(1000 * 30)
		);
}


// Update the window title.
KinkyTwits.prototype.UpdateWindowTitle = function(){
	var strTitle = document.title;
	
	// Clear out any prefix.
	strTitle = strTitle.replace(
		new RegExp( "^\\( \\d+ \\)\\s+", "i" ),
		""
		);

	// Now, check to see if we have any unread messages. We only want
	// to do this if the window is blurred.
	if (
		!this.WindowFocused &&
		this.UnreadMessageCount
		){
		
		// Add unread prefix.
		strTitle = ( "( " + this.UnreadMessageCount + " ) " + strTitle);
	
	}
		
	// Update the title.
	document.title = strTitle;
}


// This handles the window blurring.
KinkyTwits.prototype.WindowBlurHandler = function( objEvent ){
	var objSelf = this;
	
	// Update focus flag.
	this.WindowFocused = false;
}


// This handles the window focusing.
KinkyTwits.prototype.WindowFocusHandler = function( objEvent ){
	var objSelf = this;
	
	// Update focus flag.
	this.WindowFocused = true;
	
	// Reset the unread count.
	this.UnreadMessageCount = 0;
	
	// Update the title.
	this.UpdateWindowTitle();
}
	
	